@@ -23,18 +23,21 @@ import java.util.concurrent.CompletableFuture;
 
 import javax.annotation.Nullable;
 
-import org.apache.flink.annotation.VisibleForTesting;
-import org.apache.flink.runtime.checkpoint.CheckpointException;
+import com.google.common.annotations.VisibleForTesting;
 import org.apache.flink.runtime.event.AbstractEvent;
 import org.apache.flink.runtime.io.network.api.EndOfData;
 import org.apache.flink.runtime.io.network.api.EndOfPartitionEvent;
-import org.apache.flink.runtime.io.network.api.StopMode;
 import org.apache.flink.runtime.io.network.api.serialization.EventSerializer;
 import org.apache.flink.runtime.io.network.buffer.Buffer;
 import org.apache.flink.runtime.io.network.buffer.Buffer.DataType;
 import org.apache.flink.runtime.io.network.buffer.BufferCompressor;
 import org.apache.flink.runtime.io.network.buffer.BufferPool;
-import org.apache.flink.runtime.io.network.partition.*;
+import org.apache.flink.runtime.io.network.partition.BufferAvailabilityListener;
+import org.apache.flink.runtime.io.network.partition.ResultPartition;
+import org.apache.flink.runtime.io.network.partition.ResultPartitionID;
+import org.apache.flink.runtime.io.network.partition.ResultPartitionManager;
+import org.apache.flink.runtime.io.network.partition.ResultPartitionType;
+import org.apache.flink.runtime.io.network.partition.ResultSubpartitionView;
 import org.apache.flink.util.function.SupplierWithException;
 
 import org.apache.celeborn.plugin.flink.buffer.BufferWithSubpartition;
@@ -50,7 +53,7 @@ import org.apache.celeborn.plugin.flink.utils.Utils;
  */
 public class RemoteShuffleResultPartition extends ResultPartition {
 
-  private final RemoteShuffleResultPartitionDelegation delegation;
+  RemoteShuffleResultPartitionDelegation delegation;
 
   public RemoteShuffleResultPartition(
       String owningTaskName,
@@ -89,11 +92,6 @@ public class RemoteShuffleResultPartition extends ResultPartition {
         bufferPool, bufferCompressor, this::canBeCompressed, this::checkInProduceState);
   }
 
-  @Override
-  protected void setupInternal() {
-    // do not need to implement
-  }
-
   @Override
   public void emitRecord(ByteBuffer record, int targetSubpartition) throws IOException {
     delegation.emit(record, targetSubpartition, DataType.DATA_BUFFER, false);
@@ -115,12 +113,6 @@ public class RemoteShuffleResultPartition extends ResultPartition {
     }
   }
 
-  @Override
-  public void alignedBarrierTimeout(long l) {}
-
-  @Override
-  public void abortCheckpoint(long l, CheckpointException e) {}
-
   @Override
   public void finish() throws IOException {
     Utils.checkState(!isReleased(), "Result partition is already released.");
@@ -159,11 +151,6 @@ public class RemoteShuffleResultPartition extends ResultPartition {
     return 0;
   }
 
-  @Override
-  public long getSizeOfQueuedBuffersUnsafe() {
-    return 0;
-  }
-
   @Override
   public int getNumberOfQueuedBuffers(int targetSubpartition) {
     return 0;
@@ -176,9 +163,9 @@ public class RemoteShuffleResultPartition extends ResultPartition {
   }
 
   @Override
-  public void notifyEndOfData(StopMode mode) throws IOException {
+  public void notifyEndOfData() throws IOException {
     if (!delegation.isEndOfDataNotified()) {
-      broadcastEvent(new EndOfData(mode), false);
+      broadcastEvent(EndOfData.INSTANCE, false);
       delegation.setEndOfDataNotified(true);
     }
   }
@@ -210,11 +197,6 @@ public class RemoteShuffleResultPartition extends ResultPartition {
     numBuffersOut.inc(isBroadcast ? numSubpartitions : 1);
     long readableBytes =
         (long) bufferWithSubpartition.getBuffer().readableBytes() - BufferUtils.HEADER_LENGTH;
-    if (isBroadcast) {
-      resultPartitionBytes.incAll(readableBytes);
-    } else {
-      resultPartitionBytes.inc(bufferWithSubpartition.getSubpartitionIndex(), readableBytes);
-    }
     numBytesOut.inc(isBroadcast ? readableBytes * numSubpartitions : readableBytes);
   }
 }
