@@ -18,7 +18,7 @@
 package org.apache.celeborn.plugin.flink;
 
 import java.io.IOException;
-import java.net.InetSocketAddress;
+import java.net.InetAddress;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
@@ -32,8 +32,10 @@ import org.apache.flink.runtime.checkpoint.CheckpointOptions;
 import org.apache.flink.runtime.checkpoint.channel.ChannelStateWriter;
 import org.apache.flink.runtime.checkpoint.channel.InputChannelInfo;
 import org.apache.flink.runtime.checkpoint.channel.ResultSubpartitionInfo;
+import org.apache.flink.runtime.clusterframework.types.ResourceID;
 import org.apache.flink.runtime.deployment.InputGateDeploymentDescriptor;
 import org.apache.flink.runtime.event.TaskEvent;
+import org.apache.flink.runtime.executiongraph.IndexRange;
 import org.apache.flink.runtime.io.network.ConnectionID;
 import org.apache.flink.runtime.io.network.LocalConnectionManager;
 import org.apache.flink.runtime.io.network.api.CheckpointBarrier;
@@ -42,10 +44,18 @@ import org.apache.flink.runtime.io.network.buffer.BufferDecompressor;
 import org.apache.flink.runtime.io.network.buffer.BufferPool;
 import org.apache.flink.runtime.io.network.partition.ResultPartitionID;
 import org.apache.flink.runtime.io.network.partition.ResultPartitionType;
-import org.apache.flink.runtime.io.network.partition.consumer.*;
+import org.apache.flink.runtime.io.network.partition.ResultSubpartitionIndexSet;
+import org.apache.flink.runtime.io.network.partition.consumer.BufferOrEvent;
+import org.apache.flink.runtime.io.network.partition.consumer.IndexedInputGate;
+import org.apache.flink.runtime.io.network.partition.consumer.InputChannel;
+import org.apache.flink.runtime.io.network.partition.consumer.RemoteInputChannel;
+import org.apache.flink.runtime.io.network.partition.consumer.SingleInputGate;
 import org.apache.flink.runtime.jobgraph.IntermediateDataSetID;
+import org.apache.flink.runtime.taskmanager.TaskManagerLocation;
+import org.apache.flink.runtime.throughput.ThroughputCalculator;
 import org.apache.flink.util.CloseableIterator;
 import org.apache.flink.util.FlinkRuntimeException;
+import org.apache.flink.util.clock.SystemClock;
 import org.apache.flink.util.function.SupplierWithException;
 
 import org.apache.celeborn.common.CelebornConf;
@@ -53,7 +63,7 @@ import org.apache.celeborn.common.CelebornConf;
 /** A {@link IndexedInputGate} which ingest data from remote shuffle workers. */
 public class RemoteShuffleInputGate extends IndexedInputGate {
 
-  RemoteShuffleInputGateDelegation inputGateDelegation;
+  private final RemoteShuffleInputGateDelegation inputGateDelegation;
 
   public RemoteShuffleInputGate(
       CelebornConf celebornConf,
@@ -63,6 +73,7 @@ public class RemoteShuffleInputGate extends IndexedInputGate {
       SupplierWithException<BufferPool, IOException> bufferPoolFactory,
       BufferDecompressor bufferDecompressor,
       int numConcurrentReading) {
+
     inputGateDelegation =
         new RemoteShuffleInputGateDelegation(
             celebornConf,
@@ -73,8 +84,8 @@ public class RemoteShuffleInputGate extends IndexedInputGate {
             bufferDecompressor,
             numConcurrentReading,
             availabilityHelper,
-            gateDescriptor.getConsumedSubpartitionIndex(),
-            gateDescriptor.getConsumedSubpartitionIndex());
+            gateDescriptor.getConsumedSubpartitionIndexRange().getStartIndex(),
+            gateDescriptor.getConsumedSubpartitionIndexRange().getEndIndex());
   }
 
   /** Setup gate and build network connections. */
@@ -101,11 +112,6 @@ public class RemoteShuffleInputGate extends IndexedInputGate {
     return inputGateDelegation.isFinished();
   }
 
-  @Override
-  public boolean hasReceivedEndOfData() {
-    return inputGateDelegation.hasReceivedEndOfData();
-  }
-
   @Override
   public Optional<BufferOrEvent> getNext() {
     throw new UnsupportedOperationException("Not implemented (DataSet API is not supported).");
@@ -145,18 +151,25 @@ public class RemoteShuffleInputGate extends IndexedInputGate {
   }
 
   @Override
-  public int getBuffersInUseCount() {
-    return 0;
+  public void triggerDebloating() {
+    // do-nothing.
   }
 
-  @Override
-  public void announceBufferSize(int i) {}
-
   @Override
   public List<InputChannelInfo> getUnfinishedChannels() {
     return Collections.emptyList();
   }
 
+  @Override
+  public EndOfDataStatus hasReceivedEndOfData() {
+    if (inputGateDelegation.getPendingEndOfDataEvents() > 0) {
+      return EndOfDataStatus.NOT_END_OF_DATA;
+    } else {
+      // Keep compatibility with streaming mode.
+      return EndOfDataStatus.DRAINED;
+    }
+  }
+
   @Override
   public void finishReadRecoveredState() {
     // do-nothing.
@@ -197,26 +210,34 @@ public class RemoteShuffleInputGate extends IndexedInputGate {
   /** Accommodation for the incompleteness of Flink pluggable shuffle service. */
   private class FakedRemoteInputChannel extends RemoteInputChannel {
     FakedRemoteInputChannel(int channelIndex) {
+      // Flink 1.19.0
+      // [FLINK-25055][network] Support listen and notify mechanism for partition request
+      // [FLINK-33743][runtime] Support consuming multiple subpartitions on a single channel
       super(
           new SingleInputGate(
               inputGateDelegation.getTaskName(),
               inputGateDelegation.getGateIndex(),
               new IntermediateDataSetID(),
               ResultPartitionType.BLOCKING,
-              0,
               1,
               (a, b, c) -> {},
               () -> null,
               null,
               new FakedMemorySegmentProvider(),
-              0),
+              0,
+              new ThroughputCalculator(SystemClock.getInstance()),
+              null),
           channelIndex,
           new ResultPartitionID(),
-          new ConnectionID(new InetSocketAddress("", 0), 0),
+          new ResultSubpartitionIndexSet(new IndexRange(0, 0)),
+          new ConnectionID(
+              new TaskManagerLocation(ResourceID.generate(), InetAddress.getLoopbackAddress(), 1),
+              0),
           new LocalConnectionManager(),
           0,
           0,
           0,
+          0,
           new SimpleCounter(),
           new SimpleCounter(),
           new FakedChannelStateWriter());
@@ -250,6 +271,14 @@ public class RemoteShuffleInputGate extends IndexedInputGate {
     public void addOutputData(
         long cpId, ResultSubpartitionInfo info, int startSeqNum, Buffer... data) {}
 
+    @Override
+    public void addOutputDataFuture(
+        long l,
+        ResultSubpartitionInfo resultSubpartitionInfo,
+        int i,
+        CompletableFuture<List<Buffer>> completableFuture)
+        throws IllegalArgumentException {}
+
     @Override
     public void finishInput(long checkpointId) {}
 
