@@ -50,6 +50,8 @@ import org.apache.celeborn.util.HadoopUtils;
 public class MRAppMasterWithCeleborn extends MRAppMaster {
   private static final Logger logger = LoggerFactory.getLogger(MRAppMasterWithCeleborn.class);
 
+  private static final String MASTER_ENDPOINTS_ENV = "CELEBORN_MASTER_ENDPOINTS";
+
   public MRAppMasterWithCeleborn(
       ApplicationAttemptId applicationAttemptId,
       ContainerId containerId,
@@ -68,7 +70,7 @@ public class MRAppMasterWithCeleborn extends MRAppMaster {
           new LifecycleManager(applicationAttemptId.toString(), conf);
       String lmHost = lifecycleManager.getHost();
       int lmPort = lifecycleManager.getPort();
-      logger.info("RMAppMaster initialized with {} {} {}", lmHost, lmPort, applicationAttemptId);
+      logger.info("MRAppMaster initialized with {} {} {}", lmHost, lmPort, applicationAttemptId);
       JobConf lmConf = new JobConf();
       lmConf.clear();
       lmConf.set(HadoopUtils.MR_CELEBORN_LM_HOST, lmHost);
@@ -81,9 +83,9 @@ public class MRAppMasterWithCeleborn extends MRAppMaster {
   private void writeLifecycleManagerConfToTask(JobConf conf, JobConf lmConf)
       throws CelebornIOException {
     try {
-      FileSystem fs = FileSystem.get(conf);
       String jobDirStr = conf.get(MRJobConfig.MAPREDUCE_JOB_DIR);
       Path celebornConfPath = new Path(jobDirStr, HadoopUtils.MR_CELEBORN_CONF);
+      FileSystem fs = celebornConfPath.getFileSystem(conf);
 
       try (FSDataOutputStream out =
           FileSystem.create(
@@ -128,6 +130,7 @@ public class MRAppMasterWithCeleborn extends MRAppMaster {
     JobConf rmAppConf = new JobConf(new YarnConfiguration());
     rmAppConf.addResource(new Path(MRJobConfig.JOB_CONF_FILE));
     try {
+      checkJobConf(rmAppConf);
       Thread.setDefaultUncaughtExceptionHandler(new YarnUncaughtExceptionHandler());
       String containerIdStr = ensureGetSysEnv(ApplicationConstants.Environment.CONTAINER_ID.name());
       String nodeHostString = ensureGetSysEnv(ApplicationConstants.Environment.NM_HOST.name());
@@ -185,4 +188,28 @@ public class MRAppMasterWithCeleborn extends MRAppMaster {
       ExitUtil.terminate(1, t);
     }
   }
+
+  public static void checkJobConf(JobConf conf) throws IOException {
+    if (conf.getBoolean(MRJobConfig.MR_AM_JOB_RECOVERY_ENABLE, false)) {
+      logger.warn("MRAppMaster disables job recovery.");
+      // MapReduce does not set the flag which indicates whether to keep containers across
+      // application attempts in ApplicationSubmissionContext. Therefore, there is no container
+      // shared between attempts.
+      conf.setBoolean(MRJobConfig.MR_AM_JOB_RECOVERY_ENABLE, false);
+    }
+    if (conf.getFloat(MRJobConfig.COMPLETED_MAPS_FOR_REDUCE_SLOWSTART, 0.05f) != 1.0f) {
+      logger.warn("MRAppMaster disables job reduce slow start.");
+      // Make sure reduces are scheduled only after all map are completed.
+      conf.setFloat(MRJobConfig.COMPLETED_MAPS_FOR_REDUCE_SLOWSTART, 1.0f);
+    }
+    String masterEndpointsKey = HadoopUtils.MR_PREFIX + CelebornConf.MASTER_ENDPOINTS().key();
+    String masterEndpointsVal = conf.get(masterEndpointsKey);
+    if (masterEndpointsVal == null || masterEndpointsVal.isEmpty()) {
+      logger.info(
+          "MRAppMaster sets {} via environment variable {}.",
+          masterEndpointsKey,
+          MASTER_ENDPOINTS_ENV);
+      conf.set(masterEndpointsKey, ensureGetSysEnv(MASTER_ENDPOINTS_ENV));
+    }
+  }
 }
