@@ -44,6 +44,60 @@ import org.apache.celeborn.common.util.Utils;
 
 public class SortBasedPusher extends MemoryConsumer {
 
+  class MemoryThresholdManager {
+
+    private final long maxMemoryThresholdInBytes;
+    private final double smallPushTolerateFactor;
+
+    MemoryThresholdManager(double maxMemoryFactor, double smallPushTolerateFactor) {
+      this.maxMemoryThresholdInBytes = (long) (Runtime.getRuntime().maxMemory() * maxMemoryFactor);
+      this.smallPushTolerateFactor = smallPushTolerateFactor;
+    }
+
+    private boolean shouldGrow() {
+      boolean enoughSpace = pushSortMemoryThreshold <= maxMemoryThresholdInBytes;
+      double expectedPushSize = Long.MAX_VALUE;
+      if (this.expectedPushedCount != 0) {
+        expectedPushSize = this.expectedPushedBytes * 1.0 / this.expectedPushedCount;
+      }
+      boolean tooManyPushed =
+          pushedMemorySizeInBytes * 1.0 / pushedCount * (1 + this.smallPushTolerateFactor)
+              < expectedPushSize;
+      return useAdaptiveThreshold && enoughSpace && tooManyPushed;
+    }
+
+    public void growThresholdIfNeeded() {
+      if (shouldGrow()) {
+        long oldThreshold = pushSortMemoryThreshold;
+        pushSortMemoryThreshold = Math.min(pushSortMemoryThreshold * 2, maxMemoryThresholdInBytes);
+        logger.info(
+            "grow memory threshold from "
+                + Utils.bytesToString(oldThreshold)
+                + " to "
+                + Utils.bytesToString(pushSortMemoryThreshold));
+        pushedCount = 0;
+        pushedMemorySizeInBytes = 0;
+        expectedPushedBytes = 0;
+        expectedPushedCount = 0;
+      }
+    }
+
+    long pushedCount = 0;
+    long pushedMemorySizeInBytes = 0;
+
+    long expectedPushedCount = 0;
+    long expectedPushedBytes = 0;
+
+    public void updateStats(long pushedBytes, boolean updateExpected) {
+      this.pushedMemorySizeInBytes += pushedBytes;
+      this.pushedCount += 1;
+      if (updateExpected) {
+        this.expectedPushedBytes += pushedBytes;
+        this.expectedPushedCount += 1;
+      }
+    }
+  }
+
   private static final Logger logger = LoggerFactory.getLogger(SortBasedPusher.class);
 
   private static final int UAO_SIZE = UnsafeAlignedOffset.getUaoSize();
@@ -60,7 +114,7 @@ public class SortBasedPusher extends MemoryConsumer {
   private final TaskContext taskContext;
   private DataPusher dataPusher;
   private final int pushBufferMaxSize;
-  private final long pushSortMemoryThreshold;
+  private long pushSortMemoryThreshold;
   private final int shuffleId;
   private final int mapId;
   private final int attemptNumber;
@@ -72,6 +126,12 @@ public class SortBasedPusher extends MemoryConsumer {
   private int[] inversedShuffledPartitions = null;
   private final SendBufferPool sendBufferPool;
 
+  final MemoryThresholdManager memoryThresholdManager;
+
+  private final boolean useAdaptiveThreshold;
+
+  private final double maxMemoryFactor;
+
   public SortBasedPusher(
       TaskMemoryManager memoryManager,
       ShuffleClient shuffleClient,
@@ -132,14 +192,19 @@ public class SortBasedPusher extends MemoryConsumer {
     }
 
     pushBufferMaxSize = conf.clientPushBufferMaxSize();
+    useAdaptiveThreshold = conf.clientPushSortUseAdaptiveMemoryThreshold();
+    maxMemoryFactor = conf.clientPushSortMaxMemoryFactor();
     this.pushSortMemoryThreshold = pushSortMemoryThreshold;
 
+    this.memoryThresholdManager =
+        new MemoryThresholdManager(maxMemoryFactor, conf.clientPushSortSmallPushTolerateFactor());
+
     int initialSize = Math.min((int) pushSortMemoryThreshold / 8, 1024 * 1024);
     this.inMemSorter = new ShuffleInMemorySorter(this, initialSize);
     this.peakMemoryUsedBytes = getMemoryUsage();
   }
 
-  public long pushData() throws IOException {
+  public long pushData(boolean growThreshold) throws IOException {
     final ShuffleInMemorySorter.ShuffleSorterIterator sortedRecords =
         inMemSorter.getSortedIterator();
 
@@ -169,6 +234,7 @@ public class SortBasedPusher extends MemoryConsumer {
                   numPartitions);
           mapStatusLengths[currentPartition].add(bytesWritten);
           afterPush.accept(bytesWritten);
+          memoryThresholdManager.updateStats(offSet, offSet == pushBufferMaxSize);
           currentPartition = partition;
           offSet = 0;
         }
@@ -181,6 +247,7 @@ public class SortBasedPusher extends MemoryConsumer {
       if (offSet + recordSize > dataBuf.length) {
         try {
           dataPusher.addTask(partition, dataBuf, offSet);
+          memoryThresholdManager.updateStats(offSet, true);
         } catch (InterruptedException e) {
           TaskInterruptedHelper.throwTaskKillException();
         }
@@ -195,11 +262,15 @@ public class SortBasedPusher extends MemoryConsumer {
     if (offSet > 0) {
       try {
         dataPusher.addTask(currentPartition, dataBuf, offSet);
+        memoryThresholdManager.updateStats(offSet, offSet == pushBufferMaxSize);
       } catch (InterruptedException e) {
         TaskInterruptedHelper.throwTaskKillException();
       }
     }
 
+    if (growThreshold) {
+      memoryThresholdManager.growThresholdIfNeeded();
+    }
     long freedBytes = freeMemory();
     inMemSorter.freeMemory();
     taskContext.taskMetrics().incMemoryBytesSpilled(freedBytes);
@@ -218,7 +289,8 @@ public class SortBasedPusher extends MemoryConsumer {
       required = recordSize + UAO_SIZE;
     }
 
-    if (getUsed() > pushSortMemoryThreshold
+    long threshold = pushSortMemoryThreshold;
+    if (getUsed() > threshold
         && pageCursor + required > currentPage.getBaseOffset() + currentPage.size()) {
       logger.info(
           "Memory used {} exceeds threshold {}, need to trigger push. currentPage size: {}",
@@ -255,7 +327,7 @@ public class SortBasedPusher extends MemoryConsumer {
     return true;
   }
 
-  private void growPointerArrayIfNecessary() throws IOException {
+  private void growPointerArrayIfNecessary(long required) throws IOException {
     assert (inMemSorter != null);
     if (!inMemSorter.hasSpaceForAnotherRecord()) {
       if (inMemSorter.numRecords() <= 0) {
@@ -267,24 +339,50 @@ public class SortBasedPusher extends MemoryConsumer {
       }
 
       long used = inMemSorter.getMemoryUsage();
+      long requestedBytes = used / 8 * 2;
+      int allocateMemoryRetryCount = 0;
+      int maxMemoryAllocationRetry = 3;
       LongArray array = null;
-      try {
-        // could trigger spilling
-        array = allocateArray(used / 8 * 2);
-      } catch (TooLargePageException e) {
-        // The pointer array is too big to fix in a single page, spill.
-        logger.info(
-            "Pushdata in growPointerArrayIfNecessary, memory used {}",
-            Utils.bytesToString(getUsed()));
-        pushData();
-      } catch (SparkOutOfMemoryError rethrow) {
-        // should have trigger spilling
-        if (inMemSorter.numRecords() > 0) {
-          logger.error("OOM, unable to grow the pointer array");
-          throw rethrow;
+      boolean continueRetry = true;
+      while (allocateMemoryRetryCount < maxMemoryAllocationRetry && continueRetry) {
+        try {
+          // could trigger spilling
+          logger.info("asking for " + requestedBytes + " more bytes to accommodate more records");
+          array = allocateArray(requestedBytes);
+          continueRetry = false;
+        } catch (TooLargePageException e) {
+          // The pointer array is too big to fix in a single page, spill.
+          logger.info(
+              "Pushdata in growPointerArrayIfNecessary, memory used {}",
+              Utils.bytesToString(getUsed()));
+          pushData(true);
+          continueRetry = false;
+        } catch (SparkOutOfMemoryError rethrow) {
+          // should have trigger spilling
+          allocateMemoryRetryCount += 1;
+          if (inMemSorter.numRecords() > 0) {
+            if (allocateMemoryRetryCount == maxMemoryAllocationRetry) {
+              logger.error("OOM, unable to grow the pointer array");
+              throw rethrow;
+            } else {
+              long oldReq = requestedBytes;
+              requestedBytes = Math.max((long) (requestedBytes * 0.5), required);
+              logger.warn(
+                  "cannot allocate "
+                      + oldReq
+                      + " bytes, cut the request to "
+                      + requestedBytes
+                      + " bytes and retry",
+                  rethrow);
+              pushData(true);
+            }
+          } else {
+            // The new array could not be allocated, but that is not an issue as it is longer
+            // needed,
+            // as all records were spilled.
+            continueRetry = false;
+          }
         }
-        // The new array could not be allocated, but that is not an issue as it is longer needed,
-        // as all records were spilled.
       }
 
       if (inMemSorter.numRecords() <= 0) {
@@ -329,7 +427,7 @@ public class SortBasedPusher extends MemoryConsumer {
   private void allocateMemoryForRecordIfNecessary(int required) throws IOException {
     // Step 1:
     // Ensure that the pointer array has space for another record. This may cause a spill.
-    growPointerArrayIfNecessary();
+    growPointerArrayIfNecessary(required);
     // Step 2:
     // Ensure that the last page has space for another record. This may cause a spill.
     acquireNewPageIfNecessary(required);
@@ -347,7 +445,7 @@ public class SortBasedPusher extends MemoryConsumer {
     // no-op that does not allocate any memory, and therefore can't cause a spill event.
     //
     // Thus there is no need to call `acquireNewPageIfNecessary` again after this step.
-    growPointerArrayIfNecessary();
+    growPointerArrayIfNecessary(required);
   }
 
   @Override
@@ -399,15 +497,17 @@ public class SortBasedPusher extends MemoryConsumer {
     taskContext.taskMetrics().incMemoryBytesSpilled(freedBytes);
   }
 
-  public void close(boolean throwTaskKilledOnInterruption) throws IOException {
+  public long getPushSortMemoryThreshold() {
+    return this.pushSortMemoryThreshold;
+  }
+
+  public void close() throws IOException {
     cleanupResources();
     try {
       dataPusher.waitOnTermination();
-      sendBufferPool.returnPushTaskQueue(dataPusher.getAndResetIdleQueue());
+      sendBufferPool.returnPushTaskQueue(dataPusher.getIdleQueue());
     } catch (InterruptedException e) {
-      if (throwTaskKilledOnInterruption) {
-        TaskInterruptedHelper.throwTaskKillException();
-      }
+      TaskInterruptedHelper.throwTaskKillException();
     }
   }
 
