@@ -209,6 +209,27 @@ public class ShuffleClientImpl extends ShuffleClient {
     logger.info("Created ShuffleClientImpl, appUniqueId: {}", appUniqueId);
   }
 
+  protected List<TransportClientBootstrap> createBootstraps() {
+    if (authEnabled && null != lifecycleManagerRef) {
+      PbApplicationMetaRequest pbApplicationMetaRequest =
+          PbApplicationMetaRequest.newBuilder().setAppId(appUniqueId).build();
+      PbApplicationMeta pbApplicationMeta =
+          lifecycleManagerRef.askSync(
+              pbApplicationMetaRequest,
+              conf.clientRpcRegisterShuffleAskTimeout(),
+              ClassTag$.MODULE$.apply(PbApplicationMeta.class));
+      List<TransportClientBootstrap> bootstraps = Lists.newArrayList();
+      bootstraps.add(
+          new SaslClientBootstrap(
+              dataTransportConf,
+              appUniqueId,
+              new SaslCredentials(appUniqueId, pbApplicationMeta.getSecret())));
+      return Collections.unmodifiableList(bootstraps);
+    } else {
+      return Collections.emptyList();
+    }
+  }
+
   private void initDataClientFactoryIfNeeded() {
     if (dataClientFactory != null) {
       return;
@@ -220,20 +241,8 @@ public class ShuffleClientImpl extends ShuffleClient {
       logger.info("Initializing data client factory for {}.", appUniqueId);
       dataClientFactory = transportContext.createClientFactory();
     } else if (lifecycleManagerRef != null) {
-      PbApplicationMetaRequest pbApplicationMetaRequest =
-          PbApplicationMetaRequest.newBuilder().setAppId(appUniqueId).build();
-      PbApplicationMeta pbApplicationMeta =
-          lifecycleManagerRef.askSync(
-              pbApplicationMetaRequest,
-              conf.clientRpcRegisterShuffleAskTimeout(),
-              ClassTag$.MODULE$.apply(PbApplicationMeta.class));
       logger.info("Initializing data client factory for secured {}.", appUniqueId);
-      List<TransportClientBootstrap> bootstraps = Lists.newArrayList();
-      bootstraps.add(
-          new SaslClientBootstrap(
-              dataTransportConf,
-              appUniqueId,
-              new SaslCredentials(appUniqueId, pbApplicationMeta.getSecret())));
+      List<TransportClientBootstrap> bootstraps = createBootstraps();
       dataClientFactory = transportContext.createClientFactory(bootstraps);
     }
   }
@@ -505,7 +514,7 @@ public class ShuffleClientImpl extends ShuffleClient {
   }
 
   private ConcurrentHashMap<Integer, PartitionLocation> registerShuffle(
-      int shuffleId, int numMappers, int numPartitions) {
+      int shuffleId, int numMappers, int numPartitions) throws CelebornIOException {
     return registerShuffleInternal(
         shuffleId,
         numMappers,
@@ -539,18 +548,29 @@ public class ShuffleClientImpl extends ShuffleClient {
                     conf.clientRpcRegisterShuffleAskTimeout(),
                     ClassTag$.MODULE$.apply(PbRegisterShuffleResponse.class)));
 
-    if (partitionLocationMap == null) {
-      throw new CelebornIOException("Register shuffle failed for shuffle " + shuffleId);
-    }
-
     return partitionLocationMap.get(partitionId);
   }
 
   @Override
   public ConcurrentHashMap<Integer, PartitionLocation> getPartitionLocation(
-      int shuffleId, int numMappers, int numPartitions) {
-    return reducePartitionMap.computeIfAbsent(
-        shuffleId, (id) -> registerShuffle(shuffleId, numMappers, numPartitions));
+      int shuffleId, int numMappers, int numPartitions) throws CelebornIOException {
+    try {
+      return reducePartitionMap.computeIfAbsent(
+          shuffleId,
+          (id) -> {
+            try {
+              return registerShuffle(shuffleId, numMappers, numPartitions);
+            } catch (CelebornIOException e) {
+              throw new RuntimeException(e);
+            }
+          });
+    } catch (RuntimeException e) {
+      if (e.getCause() instanceof CelebornIOException) {
+        throw (CelebornIOException) e.getCause();
+      } else {
+        throw e;
+      }
+    }
   }
 
   @Override
@@ -597,8 +617,10 @@ public class ShuffleClientImpl extends ShuffleClient {
       int shuffleId,
       int numMappers,
       int numPartitions,
-      Callable<PbRegisterShuffleResponse> callable) {
+      Callable<PbRegisterShuffleResponse> callable)
+      throws CelebornIOException {
     int numRetries = registerShuffleMaxRetries;
+    StatusCode lastFailedStatusCode = null;
     while (numRetries > 0) {
       try {
         PbRegisterShuffleResponse response = callable.call();
@@ -617,16 +639,19 @@ public class ShuffleClientImpl extends ShuffleClient {
           }
           return result;
         } else if (StatusCode.SLOT_NOT_AVAILABLE.equals(respStatus)) {
+          lastFailedStatusCode = respStatus;
           logger.error(
               "LifecycleManager request slots return {}, retry again, remain retry times {}.",
               StatusCode.SLOT_NOT_AVAILABLE,
               numRetries - 1);
         } else if (StatusCode.RESERVE_SLOTS_FAILED.equals(respStatus)) {
+          lastFailedStatusCode = respStatus;
           logger.error(
               "LifecycleManager request slots return {}, retry again, remain retry times {}.",
               StatusCode.RESERVE_SLOTS_FAILED,
               numRetries - 1);
         } else {
+          lastFailedStatusCode = respStatus;
           logger.error(
               "LifecycleManager request slots return {}, retry again, remain retry times {}.",
               StatusCode.REQUEST_FAILED,
@@ -639,7 +664,7 @@ public class ShuffleClientImpl extends ShuffleClient {
             numMappers,
             numPartitions,
             e);
-        break;
+        throw new CelebornIOException("Register shuffle failed for shuffle " + shuffleId + ".", e);
       }
 
       try {
@@ -649,8 +674,8 @@ public class ShuffleClientImpl extends ShuffleClient {
       }
       numRetries--;
     }
-
-    return null;
+    throw new CelebornIOException(
+        "Register shuffle failed for shuffle " + shuffleId + ", reason: " + lastFailedStatusCode);
   }
 
   protected void limitMaxInFlight(String mapKey, PushState pushState, String hostAndPushPort)
@@ -870,10 +895,6 @@ public class ShuffleClientImpl extends ShuffleClient {
     final ConcurrentHashMap<Integer, PartitionLocation> map =
         getPartitionLocation(shuffleId, numMappers, numPartitions);
 
-    if (map == null) {
-      throw new CelebornIOException("Register shuffle failed for shuffle " + shuffleId + ".");
-    }
-
     // get location
     // If rerun or speculation task running after LifecycleManager call stageEnd,
     // register shuffle will return an empty location map, client need revive for a new location.
@@ -1648,7 +1669,15 @@ public class ShuffleClientImpl extends ShuffleClient {
   public ReduceFileGroups updateFileGroup(int shuffleId, int partitionId)
       throws CelebornIOException {
     Tuple2<ReduceFileGroups, String> fileGroupTuple =
-        reduceFileGroupsMap.computeIfAbsent(shuffleId, (id) -> loadFileGroupInternal(shuffleId));
+        reduceFileGroupsMap.compute(
+            shuffleId,
+            (id, existsTuple) -> {
+              if (existsTuple == null || existsTuple._1 == null) {
+                return loadFileGroupInternal(shuffleId);
+              } else {
+                return existsTuple;
+              }
+            });
     if (fileGroupTuple._1 == null) {
       throw new CelebornIOException(
           loadFileGroupException(shuffleId, partitionId, (fileGroupTuple._2)));
@@ -1679,7 +1708,7 @@ public class ShuffleClientImpl extends ShuffleClient {
       int[] mapAttempts,
       MetricsCallback metricsCallback)
       throws IOException {
-    if (partitionId == Utils$.MODULE$.UNKNOWN_APP_SHUFFLE_ID()) {
+    if (shuffleId == Utils$.MODULE$.UNKNOWN_APP_SHUFFLE_ID()) {
       logger.warn("Shuffle data is empty for shuffle {}: UNKNOWN_APP_SHUFFLE_ID.", shuffleId);
       return CelebornInputStream.empty();
     }
@@ -1755,6 +1784,7 @@ public class ShuffleClientImpl extends ShuffleClient {
 
   @Override
   public void setupLifecycleManagerRef(String host, int port) {
+    logger.info("setupLifecycleManagerRef: host = {}, port = {}", host, port);
     lifecycleManagerRef =
         rpcEnv.setupEndpointRef(new RpcAddress(host, port), RpcNameConstants.LIFECYCLE_MANAGER_EP);
     initDataClientFactoryIfNeeded();
