@@ -17,7 +17,9 @@
 
 package org.apache.celeborn.client
 
+import java.lang.{Byte => JByte}
 import java.nio.ByteBuffer
+import java.security.SecureRandom
 import java.util
 import java.util.{function, List => JList}
 import java.util.concurrent.{Callable, ConcurrentHashMap, LinkedBlockingQueue, ScheduledFuture, TimeUnit}
@@ -40,12 +42,14 @@ import org.apache.celeborn.common.CelebornConf
 import org.apache.celeborn.common.client.MasterClient
 import org.apache.celeborn.common.identity.{IdentityProvider, UserIdentifier}
 import org.apache.celeborn.common.internal.Logging
-import org.apache.celeborn.common.meta.{ShufflePartitionLocationInfo, WorkerInfo}
+import org.apache.celeborn.common.meta.{ApplicationMeta, ShufflePartitionLocationInfo, WorkerInfo}
+import org.apache.celeborn.common.network.sasl.registration.RegistrationInfo
 import org.apache.celeborn.common.protocol._
 import org.apache.celeborn.common.protocol.RpcNameConstants.WORKER_EP
 import org.apache.celeborn.common.protocol.message.ControlMessages._
 import org.apache.celeborn.common.protocol.message.StatusCode
 import org.apache.celeborn.common.rpc._
+import org.apache.celeborn.common.rpc.{ClientSaslContextBuilder, RpcSecurityContext, RpcSecurityContextBuilder}
 import org.apache.celeborn.common.rpc.netty.{LocalNettyRpcCallContext, RemoteNettyRpcCallContext}
 import org.apache.celeborn.common.util.{JavaUtils, PbSerDeUtils, ThreadUtils, Utils}
 // Can Remove this if celeborn don't support scala211 in future
@@ -108,7 +112,8 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     .build().asInstanceOf[Cache[Int, ByteBuffer]]
 
   private val mockDestroyFailure = conf.testMockDestroySlotsFailure
-
+  private val authEnabled = conf.authEnabledOnClient
+  private var applicationMeta: ApplicationMeta = _
   @VisibleForTesting
   def workerSnapshots(shuffleId: Int): util.Map[WorkerInfo, ShufflePartitionLocationInfo] =
     shuffleAllocatedWorkers.get(shuffleId)
@@ -155,14 +160,44 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
   // init driver celeborn LifecycleManager rpc service
   override val rpcEnv: RpcEnv = RpcEnv.create(
     RpcNameConstants.LIFECYCLE_MANAGER_SYS,
+    TransportModuleConstants.RPC_LIFECYCLEMANAGER_MODULE,
     lifecycleHost,
     conf.shuffleManagerPort,
-    conf)
+    conf,
+    None)
   rpcEnv.setupEndpoint(RpcNameConstants.LIFECYCLE_MANAGER_EP, this)
 
   logInfo(s"Starting LifecycleManager on ${rpcEnv.address}")
 
-  private val masterClient = new MasterClient(rpcEnv, conf)
+  private var masterRpcEnvInUse = rpcEnv
+  private var workerRpcEnvInUse = rpcEnv
+  if (authEnabled) {
+    logInfo(s"Authentication is enabled; setting up master and worker RPC environments")
+    val appSecret = createSecret()
+    applicationMeta = ApplicationMeta(appUniqueId, appSecret)
+    val registrationInfo = new RegistrationInfo()
+    masterRpcEnvInUse =
+      RpcEnv.create(
+        RpcNameConstants.LIFECYCLE_MANAGER_MASTER_SYS,
+        TransportModuleConstants.RPC_SERVICE_MODULE,
+        lifecycleHost,
+        0,
+        conf,
+        createRpcSecurityContext(
+          appSecret,
+          addClientRegistrationBootstrap = true,
+          Some(registrationInfo)))
+    workerRpcEnvInUse =
+      RpcEnv.create(
+        RpcNameConstants.LIFECYCLE_MANAGER_WORKER_SYS,
+        TransportModuleConstants.RPC_SERVICE_MODULE,
+        lifecycleHost,
+        0,
+        conf,
+        createRpcSecurityContext(appSecret))
+  }
+
+  private val masterClient = new MasterClient(masterRpcEnvInUse, conf, false)
   val commitManager = new CommitManager(appUniqueId, conf, this)
   val workerStatusTracker = new WorkerStatusTracker(conf, this)
   private val heartbeater =
@@ -217,6 +252,36 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
       rpcEnv.shutdown()
       rpcEnv.awaitTermination()
     }
+    if (authEnabled) {
+      if (masterRpcEnvInUse != null) {
+        masterRpcEnvInUse.shutdown()
+        masterRpcEnvInUse.awaitTermination()
+      }
+      if (workerRpcEnvInUse != null) {
+        workerRpcEnvInUse.shutdown()
+        workerRpcEnvInUse.awaitTermination()
+      }
+    }
+  }
+
+  /**
+   * Creates security context for external RPC endpoint.
+   */
+  def createRpcSecurityContext(
+      appSecret: String,
+      addClientRegistrationBootstrap: Boolean = false,
+      registrationInfo: Option[RegistrationInfo] = None): Option[RpcSecurityContext] = {
+    val clientSaslContextBuilder = new ClientSaslContextBuilder()
+      .withAddRegistrationBootstrap(addClientRegistrationBootstrap)
+      .withAppId(appUniqueId)
+      .withSaslUser(appUniqueId)
+      .withSaslPassword(appSecret)
+    if (registrationInfo.isDefined) {
+      clientSaslContextBuilder.withRegistrationInfo(registrationInfo.get)
+    }
+    val rpcSecurityContext = new RpcSecurityContextBuilder()
+      .withClientSaslContext(clientSaslContextBuilder.build()).build()
+    Some(rpcSecurityContext)
   }
 
   def getUserIdentifier: UserIdentifier = {
@@ -351,6 +416,15 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
       val shuffleId = pb.getShuffleId
       logDebug(s"Received ReportShuffleFetchFailure request, appShuffleId $appShuffleId shuffleId $shuffleId")
       handleReportShuffleFetchFailure(context, appShuffleId, shuffleId)
+
+    case pb: PbApplicationMetaRequest =>
+      logDebug(s"Received request for meta info ${pb.getAppId}.")
+      if (applicationMeta == null) {
+        context.sendFailure(
+          new IllegalArgumentException("Application meta is not initialized for this app."))
+      } else {
+        context.reply(PbSerDeUtils.toPbApplicationMeta(applicationMeta))
+      }
   }
 
   def setupEndpoints(
@@ -359,7 +433,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
       connectFailedWorkers: ShuffleFailedWorkers): Unit = {
     val futures = new util.LinkedList[(Future[RpcEndpointRef], WorkerInfo)]()
     slots.asScala foreach { case (workerInfo, _) =>
-      val future = rpcEnv.asyncSetupEndpointRefByAddr(RpcEndpointAddress(
+      val future = workerRpcEnvInUse.asyncSetupEndpointRefByAddr(RpcEndpointAddress(
         RpcAddress.apply(workerInfo.host, workerInfo.rpcPort),
         WORKER_EP))
       futures.add((future, workerInfo))
@@ -511,16 +585,16 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
           case _ => Option.empty
         }
 
+        val locations = PbSerDeUtils.fromPbPackedPartitionLocationsPair(
+          response.getPackedPartitionLocationsPair)._1.asScala
+
         registeringShuffleRequest.asScala
           .get(shuffleId)
           .foreach(_.asScala.foreach(context => {
             partitionType match {
               case PartitionType.MAP =>
                 if (response.getStatus == StatusCode.SUCCESS.getValue) {
-                  val partitionLocations =
-                    response.getPartitionLocationsList.asScala.filter(
-                      _.getId == context.partitionId).map(r =>
-                      PbSerDeUtils.fromPbPartitionLocation(r)).toArray
+                  val partitionLocations = locations.filter(_.getId == context.partitionId).toArray
                   processMapTaskReply(
                     shuffleId,
                     context.context,
@@ -563,7 +637,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
         replyRegisterShuffle(RegisterShuffleResponse(StatusCode.SLOT_NOT_AVAILABLE, Array.empty))
         return
       case StatusCode.SUCCESS =>
-        logDebug(s"OfferSlots for $shuffleId Success!Slots Info: ${res.workerResource}")
+        logInfo(s"OfferSlots for $shuffleId Success!Slots Info: ${res.workerResource}")
       case StatusCode.WORKER_EXCLUDED =>
         logInfo(s"OfferSlots for $shuffleId failed due to all workers be excluded!")
         replyRegisterShuffle(RegisterShuffleResponse(StatusCode.WORKER_EXCLUDED, Array.empty))
@@ -743,8 +817,8 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
       return
     }
 
-    def areAllMapTasksEnd(shuffleId: Int): Boolean = {
-      ClientUtils.areAllMapperAttemptsFinished(commitManager.getMapperAttempts(shuffleId))
+    def isAllMaptaskEnd(shuffleId: Int): Boolean = {
+      !commitManager.getMapperAttempts(shuffleId).exists(_ < 0)
     }
 
     shuffleIds.synchronized {
@@ -781,7 +855,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
                 s"unexpected! unknown appShuffleId $appShuffleId when checking shuffle deterministic level"))
         }
       } else {
-        shuffleIds.values.map(v => v._1).toSeq.reverse.find(areAllMapTasksEnd) match {
+        shuffleIds.values.map(v => v._1).toSeq.reverse.find(isAllMaptaskEnd) match {
           case Some(shuffleId) =>
             val pbGetShuffleIdResponse = {
               logDebug(
@@ -1072,7 +1146,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
               s" ${destroyWorkerInfo.readableAddress()}, init according to partition info")
             try {
               if (!workerStatusTracker.workerExcluded(destroyWorkerInfo)) {
-                destroyWorkerInfo.endpoint = rpcEnv.setupEndpointRef(
+                destroyWorkerInfo.endpoint = workerRpcEnvInUse.setupEndpointRef(
                   RpcAddress.apply(destroyWorkerInfo.host, destroyWorkerInfo.rpcPort),
                   WORKER_EP)
               } else {
@@ -1455,6 +1529,8 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
       } else {
         Set.empty[WorkerInfo]
       }
+    // UserResourceConsumption and DiskInfo are eliminated from WorkerInfo
+    // during serialization of RequestSlots
     val req =
       RequestSlots(
         appUniqueId,
@@ -1466,7 +1542,8 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
         userIdentifier,
         slotsAssignMaxWorkers,
         availableStorageTypes,
-        excludedWorkerSet)
+        excludedWorkerSet,
+        true)
     val res = requestMasterRequestSlots(req)
     if (res.status != StatusCode.SUCCESS) {
       requestMasterRequestSlots(req)
@@ -1482,7 +1559,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     } catch {
       case e: Exception =>
         logError(s"AskSync RegisterShuffle for $shuffleKey failed.", e)
-        RequestSlotsResponse(StatusCode.REQUEST_FAILED, new WorkerResource())
+        RequestSlotsResponse(StatusCode.REQUEST_FAILED, new WorkerResource(), message.packed)
     }
   }
 
@@ -1580,4 +1657,12 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     heartbeater.stop()
     super.stop()
   }
+
+  private def createSecret(): String = {
+    val bits = 256
+    val rnd = new SecureRandom()
+    val secretBytes = new Array[Byte](bits / JByte.SIZE)
+    rnd.nextBytes(secretBytes)
+    JavaUtils.bytesToString(ByteBuffer.wrap(secretBytes))
+  }
 }
