@@ -18,7 +18,7 @@
 package org.apache.celeborn.common.protocol.message
 
 import java.util
-import java.util.UUID
+import java.util.{Collections, UUID}
 
 import scala.collection.JavaConverters._
 
@@ -26,7 +26,7 @@ import org.roaringbitmap.RoaringBitmap
 
 import org.apache.celeborn.common.identity.UserIdentifier
 import org.apache.celeborn.common.internal.Logging
-import org.apache.celeborn.common.meta.{DiskInfo, WorkerInfo}
+import org.apache.celeborn.common.meta.{DiskInfo, WorkerInfo, WorkerStatus}
 import org.apache.celeborn.common.network.protocol.TransportMessage
 import org.apache.celeborn.common.protocol._
 import org.apache.celeborn.common.protocol.MessageType._
@@ -86,6 +86,8 @@ object ControlMessages extends Logging {
         pushPort: Int,
         fetchPort: Int,
         replicatePort: Int,
+        internalPort: Int,
+        networkLocation: String,
         disks: Map[String, DiskInfo],
         userResourceConsumption: Map[UserIdentifier, ResourceConsumption],
         requestId: String): PbRegisterWorker = {
@@ -98,6 +100,8 @@ object ControlMessages extends Logging {
         .setPushPort(pushPort)
         .setFetchPort(fetchPort)
         .setReplicatePort(replicatePort)
+        .setInternalPort(internalPort)
+        .setNetworkLocation(networkLocation)
         .addAllDisks(pbDisks)
         .putAllUserResourceConsumption(pbUserResourceConsumption)
         .setRequestId(requestId)
@@ -116,11 +120,14 @@ object ControlMessages extends Logging {
       activeShuffleKeys: util.Set[String],
       estimatedAppDiskUsage: util.HashMap[String, java.lang.Long],
       highWorkload: Boolean,
+      workerStatus: WorkerStatus,
       override var requestId: String = ZERO_UUID) extends MasterRequestMessage
 
   case class HeartbeatFromWorkerResponse(
       expiredShuffleKeys: util.HashSet[String],
-      registered: Boolean) extends MasterMessage
+      registered: Boolean,
+      workerEvent: WorkerEventType = WorkerEventType.None)
+    extends MasterMessage
 
   object RegisterShuffle {
     def apply(
@@ -153,12 +160,13 @@ object ControlMessages extends Logging {
   object RegisterShuffleResponse {
     def apply(
         status: StatusCode,
-        partitionLocations: Array[PartitionLocation]): PbRegisterShuffleResponse =
-      PbRegisterShuffleResponse.newBuilder()
+        partitionLocations: Array[PartitionLocation]): PbRegisterShuffleResponse = {
+      val builder = PbRegisterShuffleResponse.newBuilder()
         .setStatus(status.getValue)
-        .addAllPartitionLocations(
-          partitionLocations.map(PbSerDeUtils.toPbPartitionLocation).toSeq.asJava)
-        .build()
+      builder.setPackedPartitionLocationsPair(
+        PbSerDeUtils.toPbPackedPartitionLocationsPair(partitionLocations.toList))
+      builder.build()
+    }
   }
 
   case class RequestSlots(
@@ -172,6 +180,7 @@ object ControlMessages extends Logging {
       maxWorkers: Int,
       availableStorageTypes: Int,
       excludedWorkerSet: Set[WorkerInfo] = Set.empty,
+      packed: Boolean = false,
       override var requestId: String = ZERO_UUID)
     extends MasterRequestMessage
 
@@ -192,7 +201,8 @@ object ControlMessages extends Logging {
 
   case class RequestSlotsResponse(
       status: StatusCode,
-      workerResource: WorkerResource)
+      workerResource: WorkerResource,
+      packed: Boolean = false)
     extends MasterMessage
 
   object Revive {
@@ -272,7 +282,7 @@ object ControlMessages extends Logging {
       status: StatusCode,
       fileGroup: util.Map[Integer, util.Set[PartitionLocation]],
       attempts: Array[Int],
-      partitionIds: util.Set[Integer] = new util.HashSet[Integer]())
+      partitionIds: util.Set[Integer] = Collections.emptySet[Integer]())
     extends MasterMessage
 
   object WorkerExclude {
@@ -281,10 +291,10 @@ object ControlMessages extends Logging {
         workersToRemove: util.List[WorkerInfo],
         requestId: String): PbWorkerExclude = PbWorkerExclude.newBuilder()
       .addAllWorkersToAdd(workersToAdd.asScala.map { workerInfo =>
-        PbSerDeUtils.toPbWorkerInfo(workerInfo, true)
+        PbSerDeUtils.toPbWorkerInfo(workerInfo, true, false)
       }.toList.asJava)
       .addAllWorkersToRemove(workersToRemove.asScala.map { workerInfo =>
-        PbSerDeUtils.toPbWorkerInfo(workerInfo, true)
+        PbSerDeUtils.toPbWorkerInfo(workerInfo, true, false)
       }.toList.asJava)
       .setRequestId(requestId)
       .build()
@@ -393,7 +403,21 @@ object ControlMessages extends Logging {
       PbRemoveWorkersUnavailableInfo.newBuilder()
         .setRequestId(requestId)
         .addAllWorkerInfo(unavailable.asScala.map { workerInfo =>
-          PbSerDeUtils.toPbWorkerInfo(workerInfo, true)
+          PbSerDeUtils.toPbWorkerInfo(workerInfo, true, false)
+        }.toList.asJava)
+        .build()
+  }
+
+  object WorkerEventRequest {
+    def apply(
+        workers: util.List[WorkerInfo],
+        eventType: String,
+        requestId: String): PbWorkerEventRequest =
+      PbWorkerEventRequest.newBuilder()
+        .setRequestId(requestId)
+        .setWorkerEventType(WorkerEventType.valueOf(eventType))
+        .addAllWorkers(workers.asScala.map { workerInfo =>
+          PbSerDeUtils.toPbWorkerInfo(workerInfo, true, false)
         }.toList.asJava)
         .build()
   }
@@ -513,6 +537,7 @@ object ControlMessages extends Logging {
           activeShuffleKeys,
           estimatedAppDiskUsage,
           highWorkload,
+          workerStatus,
           requestId) =>
       val pbDisks = disks.map(PbSerDeUtils.toPbDiskInfo).asJava
       val pbUserResourceConsumption =
@@ -528,14 +553,16 @@ object ControlMessages extends Logging {
         .addAllActiveShuffleKeys(activeShuffleKeys)
         .putAllEstimatedAppDiskUsage(estimatedAppDiskUsage)
         .setHighWorkload(highWorkload)
+        .setWorkerStatus(PbSerDeUtils.toPbWorkerStatus(workerStatus))
         .setRequestId(requestId)
         .build().toByteArray
       new TransportMessage(MessageType.HEARTBEAT_FROM_WORKER, payload)
 
-    case HeartbeatFromWorkerResponse(expiredShuffleKeys, registered) =>
+    case HeartbeatFromWorkerResponse(expiredShuffleKeys, registered, workerEventType) =>
       val payload = PbHeartbeatFromWorkerResponse.newBuilder()
         .addAllExpiredShuffleKeys(expiredShuffleKeys)
         .setRegistered(registered)
+        .setWorkerEventType(workerEventType)
         .build().toByteArray
       new TransportMessage(MessageType.HEARTBEAT_FROM_WORKER_RESPONSE, payload)
 
@@ -559,6 +586,7 @@ object ControlMessages extends Logging {
           maxWorkers,
           availableStorageTypes,
           excludedWorkerSet,
+          packed,
           requestId) =>
       val payload = PbRequestSlots.newBuilder()
         .setApplicationId(applicationId)
@@ -571,7 +599,9 @@ object ControlMessages extends Logging {
         .setRequestId(requestId)
         .setAvailableStorageTypes(availableStorageTypes)
         .setUserIdentifier(PbSerDeUtils.toPbUserIdentifier(userIdentifier))
-        .addAllExcludedWorkerSet(excludedWorkerSet.map(PbSerDeUtils.toPbWorkerInfo(_, true)).asJava)
+        .addAllExcludedWorkerSet(excludedWorkerSet.map(
+          PbSerDeUtils.toPbWorkerInfo(_, true, true)).asJava)
+        .setPacked(packed)
         .build().toByteArray
       new TransportMessage(MessageType.REQUEST_SLOTS, payload)
 
@@ -592,12 +622,16 @@ object ControlMessages extends Logging {
         .setStatus(status.getValue).build().toByteArray
       new TransportMessage(MessageType.RELEASE_SLOTS_RESPONSE, payload)
 
-    case RequestSlotsResponse(status, workerResource) =>
+    case RequestSlotsResponse(status, workerResource, packed) =>
       val builder = PbRequestSlotsResponse.newBuilder()
         .setStatus(status.getValue)
       if (!workerResource.isEmpty) {
-        builder.putAllWorkerResource(
-          PbSerDeUtils.toPbWorkerResource(workerResource))
+        if (packed) {
+          builder.putAllPackedWorkerResource(PbSerDeUtils.toPbPackedWorkerResource(workerResource))
+        } else {
+          builder.putAllWorkerResource(
+            PbSerDeUtils.toPbWorkerResource(workerResource))
+        }
       }
       val payload = builder.build().toByteArray
       new TransportMessage(MessageType.REQUEST_SLOTS_RESPONSE, payload)
@@ -636,10 +670,10 @@ object ControlMessages extends Logging {
         .setStatus(status.getValue)
       builder.putAllFileGroups(
         fileGroup.asScala.map { case (partitionId, fileGroup) =>
-          (
-            partitionId,
-            PbFileGroup.newBuilder().addAllLocations(fileGroup.asScala.map(PbSerDeUtils
-              .toPbPartitionLocation).toList.asJava).build())
+          val pbFileGroupBuilder = PbFileGroup.newBuilder()
+          pbFileGroupBuilder.setPartitionLocationsPair(
+            PbSerDeUtils.toPbPackedPartitionLocationsPair(fileGroup.asScala.toList))
+          (partitionId, pbFileGroupBuilder.build())
         }.asJava)
       builder.addAllAttempts(attempts.map(Integer.valueOf).toIterable.asJava)
       builder.addAllPartitionIds(partitionIds)
@@ -700,7 +734,7 @@ object ControlMessages extends Logging {
         .setTotalWritten(totalWritten)
         .setFileCount(fileCount)
         .addAllNeedCheckedWorkerList(needCheckedWorkerList.asScala.map(
-          PbSerDeUtils.toPbWorkerInfo(_, true)).toList.asJava)
+          PbSerDeUtils.toPbWorkerInfo(_, true, true)).toList.asJava)
         .setShouldResponse(shouldResponse)
         .build().toByteArray
       new TransportMessage(MessageType.HEARTBEAT_FROM_APPLICATION, payload)
@@ -713,11 +747,11 @@ object ControlMessages extends Logging {
       val payload = PbHeartbeatFromApplicationResponse.newBuilder()
         .setStatus(statusCode.getValue)
         .addAllExcludedWorkers(
-          excludedWorkers.asScala.map(PbSerDeUtils.toPbWorkerInfo(_, true)).toList.asJava)
+          excludedWorkers.asScala.map(PbSerDeUtils.toPbWorkerInfo(_, true, true)).toList.asJava)
         .addAllUnknownWorkers(
-          unknownWorkers.asScala.map(PbSerDeUtils.toPbWorkerInfo(_, true)).toList.asJava)
+          unknownWorkers.asScala.map(PbSerDeUtils.toPbWorkerInfo(_, true, true)).toList.asJava)
         .addAllShuttingWorkers(
-          shuttingWorkers.asScala.map(PbSerDeUtils.toPbWorkerInfo(_, true)).toList.asJava)
+          shuttingWorkers.asScala.map(PbSerDeUtils.toPbWorkerInfo(_, true, true)).toList.asJava)
         .build().toByteArray
       new TransportMessage(MessageType.HEARTBEAT_FROM_APPLICATION_RESPONSE, payload)
 
@@ -738,7 +772,7 @@ object ControlMessages extends Logging {
     case ReportWorkerUnavailable(failed, requestId) =>
       val payload = PbReportWorkerUnavailable.newBuilder()
         .addAllUnavailable(failed.asScala.map { workerInfo =>
-          PbSerDeUtils.toPbWorkerInfo(workerInfo, true)
+          PbSerDeUtils.toPbWorkerInfo(workerInfo, true, false)
         }
           .toList.asJava)
         .setRequestId(requestId).build().toByteArray
@@ -747,6 +781,12 @@ object ControlMessages extends Logging {
     case pb: PbRemoveWorkersUnavailableInfo =>
       new TransportMessage(MessageType.REMOVE_WORKERS_UNAVAILABLE_INFO, pb.toByteArray)
 
+    case pb: PbWorkerEventRequest =>
+      new TransportMessage(MessageType.WORKER_EVENT_REQUEST, pb.toByteArray)
+
+    case pb: PbWorkerEventResponse =>
+      new TransportMessage(MessageType.WORKER_EVENT_RESPONSE, pb.toByteArray)
+
     case pb: PbRegisterWorkerResponse =>
       new TransportMessage(MessageType.REGISTER_WORKER_RESPONSE, pb.toByteArray)
 
@@ -765,10 +805,8 @@ object ControlMessages extends Logging {
       val payload = PbReserveSlots.newBuilder()
         .setApplicationId(applicationId)
         .setShuffleId(shuffleId)
-        .addAllPrimaryLocations(primaryLocations.asScala
-          .map(PbSerDeUtils.toPbPartitionLocation).toList.asJava)
-        .addAllReplicaLocations(replicaLocations.asScala
-          .map(PbSerDeUtils.toPbPartitionLocation).toList.asJava)
+        .setPartitionLocationsPair(PbSerDeUtils.toPbPackedPartitionLocationsPair(
+          primaryLocations.asScala.toList ++ replicaLocations.asScala.toList))
         .setSplitThreshold(splitThreshold)
         .setSplitMode(splitMode.getValue)
         .setPartitionType(partType.getValue)
@@ -861,6 +899,12 @@ object ControlMessages extends Logging {
 
     case pb: PbCheckWorkersAvailableResponse =>
       new TransportMessage(MessageType.CHECK_WORKERS_AVAILABLE_RESPONSE, pb.toByteArray)
+
+    case pb: PbApplicationMeta =>
+      new TransportMessage(MessageType.APPLICATION_META, pb.toByteArray)
+
+    case pb: PbApplicationMetaRequest =>
+      new TransportMessage(MessageType.APPLICATION_META_REQUEST, pb.toByteArray)
   }
 
   // TODO change return type to GeneratedMessageV3
@@ -910,6 +954,9 @@ object ControlMessages extends Logging {
         if (!pbHeartbeatFromWorker.getActiveShuffleKeysList.isEmpty) {
           activeShuffleKeys.addAll(pbHeartbeatFromWorker.getActiveShuffleKeysList)
         }
+
+        val workerStatus = PbSerDeUtils.fromPbWorkerStatus(pbHeartbeatFromWorker.getWorkerStatus)
+
         HeartbeatFromWorker(
           pbHeartbeatFromWorker.getHost,
           pbHeartbeatFromWorker.getRpcPort,
@@ -921,6 +968,7 @@ object ControlMessages extends Logging {
           activeShuffleKeys,
           estimatedAppDiskUsage,
           pbHeartbeatFromWorker.getHighWorkload,
+          workerStatus,
           pbHeartbeatFromWorker.getRequestId)
 
       case HEARTBEAT_FROM_WORKER_RESPONSE_VALUE =>
@@ -930,7 +978,11 @@ object ControlMessages extends Logging {
         if (pbHeartbeatFromWorkerResponse.getExpiredShuffleKeysCount > 0) {
           expiredShuffleKeys.addAll(pbHeartbeatFromWorkerResponse.getExpiredShuffleKeysList)
         }
-        HeartbeatFromWorkerResponse(expiredShuffleKeys, pbHeartbeatFromWorkerResponse.getRegistered)
+
+        HeartbeatFromWorkerResponse(
+          expiredShuffleKeys,
+          pbHeartbeatFromWorkerResponse.getRegistered,
+          pbHeartbeatFromWorkerResponse.getWorkerEventType)
 
       case REGISTER_SHUFFLE_VALUE =>
         PbRegisterShuffle.parseFrom(message.getPayload)
@@ -957,14 +1009,22 @@ object ControlMessages extends Logging {
           pbRequestSlots.getMaxWorkers,
           pbRequestSlots.getAvailableStorageTypes,
           excludedWorkerInfoSet,
+          pbRequestSlots.getPacked,
           pbRequestSlots.getRequestId)
 
       case REQUEST_SLOTS_RESPONSE_VALUE =>
         val pbRequestSlotsResponse = PbRequestSlotsResponse.parseFrom(message.getPayload)
+        val workerResource =
+          if (pbRequestSlotsResponse.getWorkerResourceMap.isEmpty) {
+            PbSerDeUtils.fromPbPackedWorkerResource(
+              pbRequestSlotsResponse.getPackedWorkerResourceMap)
+          } else {
+            PbSerDeUtils.fromPbWorkerResource(
+              pbRequestSlotsResponse.getWorkerResourceMap)
+          }
         RequestSlotsResponse(
           Utils.toStatusCode(pbRequestSlotsResponse.getStatus),
-          PbSerDeUtils.fromPbWorkerResource(
-            pbRequestSlotsResponse.getWorkerResourceMap))
+          workerResource)
 
       case CHANGE_LOCATION_VALUE =>
         PbRevive.parseFrom(message.getPayload)
@@ -997,8 +1057,8 @@ object ControlMessages extends Logging {
           case (partitionId, fileGroup) =>
             (
               partitionId,
-              fileGroup.getLocationsList.asScala.map(
-                PbSerDeUtils.fromPbPartitionLocation).toSet.asJava)
+              PbSerDeUtils.fromPbPackedPartitionLocationsPair(
+                fileGroup.getPartitionLocationsPair)._1.asScala.toSet.asJava)
         }.asJava
 
         val attempts = pbGetReducerFileGroupResponse.getAttemptsList.asScala.map(_.toInt).toArray
@@ -1083,16 +1143,31 @@ object ControlMessages extends Logging {
       case REGISTER_WORKER_RESPONSE_VALUE =>
         PbRegisterWorkerResponse.parseFrom(message.getPayload)
 
+      case WORKER_EVENT_REQUEST_VALUE =>
+        PbWorkerEventRequest.parseFrom(message.getPayload)
+
+      case WORKER_EVENT_RESPONSE_VALUE =>
+        PbWorkerEventResponse.parseFrom(message.getPayload)
+
       case RESERVE_SLOTS_VALUE =>
         val pbReserveSlots = PbReserveSlots.parseFrom(message.getPayload)
         val userIdentifier = PbSerDeUtils.fromPbUserIdentifier(pbReserveSlots.getUserIdentifier)
+        val (primaryLocations, replicateLocations) =
+          if (pbReserveSlots.getPrimaryLocationsList.isEmpty) {
+            PbSerDeUtils.fromPbPackedPartitionLocationsPair(
+              pbReserveSlots.getPartitionLocationsPair)
+          } else {
+            (
+              new util.ArrayList[PartitionLocation](pbReserveSlots.getPrimaryLocationsList.asScala
+                .map(PbSerDeUtils.fromPbPartitionLocation).toList.asJava),
+              new util.ArrayList[PartitionLocation](pbReserveSlots.getReplicaLocationsList.asScala
+                .map(PbSerDeUtils.fromPbPartitionLocation).toList.asJava))
+          }
         ReserveSlots(
           pbReserveSlots.getApplicationId,
           pbReserveSlots.getShuffleId,
-          new util.ArrayList[PartitionLocation](pbReserveSlots.getPrimaryLocationsList.asScala
-            .map(PbSerDeUtils.fromPbPartitionLocation).toList.asJava),
-          new util.ArrayList[PartitionLocation](pbReserveSlots.getReplicaLocationsList.asScala
-            .map(PbSerDeUtils.fromPbPartitionLocation).toList.asJava),
+          primaryLocations,
+          replicateLocations,
           pbReserveSlots.getSplitThreshold,
           Utils.toShuffleSplitMode(pbReserveSlots.getSplitMode),
           Utils.toPartitionType(pbReserveSlots.getPartitionType),
@@ -1194,6 +1269,12 @@ object ControlMessages extends Logging {
 
       case CHECK_WORKERS_AVAILABLE_RESPONSE_VALUE =>
         PbCheckWorkersAvailableResponse.parseFrom(message.getPayload)
+
+      case APPLICATION_META_VALUE =>
+        PbApplicationMeta.parseFrom(message.getPayload)
+
+      case APPLICATION_META_REQUEST_VALUE =>
+        PbApplicationMetaRequest.parseFrom(message.getPayload)
     }
   }
 }
