@@ -24,19 +24,19 @@ import java.util.concurrent._
 import java.util.concurrent.atomic.AtomicBoolean
 import javax.annotation.Nullable
 
+import scala.collection.JavaConverters.seqAsJavaListConverter
 import scala.concurrent.{Future, Promise}
 import scala.reflect.ClassTag
 import scala.util.{DynamicVariable, Failure, Success}
 import scala.util.control.NonFatal
 
-import com.google.common.base.Throwables
-
 import org.apache.celeborn.common.CelebornConf
 import org.apache.celeborn.common.internal.Logging
 import org.apache.celeborn.common.network.TransportContext
-import org.apache.celeborn.common.network.buffer.NioManagedBuffer
 import org.apache.celeborn.common.network.client._
-import org.apache.celeborn.common.network.protocol.{RequestMessage => NRequestMessage, RpcFailure => NRpcFailure, RpcRequest}
+import org.apache.celeborn.common.network.protocol.{RequestMessage => NRequestMessage, RpcRequest}
+import org.apache.celeborn.common.network.sasl.{SaslClientBootstrap, SaslServerBootstrap}
+import org.apache.celeborn.common.network.sasl.registration.{RegistrationClientBootstrap, RegistrationServerBootstrap}
 import org.apache.celeborn.common.network.server._
 import org.apache.celeborn.common.protocol.{RpcNameConstants, TransportModuleConstants}
 import org.apache.celeborn.common.rpc._
@@ -48,20 +48,42 @@ class NettyRpcEnv(
     javaSerializerInstance: JavaSerializerInstance) extends RpcEnv(config) with Logging {
 
   val celebornConf = config.conf
+  val securityContext = config.securityContext
 
   private[celeborn] val transportConf = Utils.fromCelebornConf(
     celebornConf.clone,
-    TransportModuleConstants.RPC_MODULE,
+    config.transportModule,
     celebornConf.rpcIoThreads.getOrElse(config.numUsableCores))
 
   private val dispatcher: Dispatcher = new Dispatcher(this)
 
   private var worker: RpcEndpoint = null
 
-  private val transportContext =
+  // Visible for tests
+  private[netty] val transportContext =
     new TransportContext(transportConf, new NettyRpcHandler(dispatcher, this))
 
-  val clientFactory = transportContext.createClientFactory()
+  private def createClientBootstraps(): java.util.List[TransportClientBootstrap] = {
+    val bootstrapOpt = securityContext.flatMap(_.clientSaslContext.map { clientSaslContext =>
+      if (clientSaslContext.addRegistrationBootstrap) {
+        logInfo("Add registration client bootstrap")
+        new RegistrationClientBootstrap(
+          transportConf,
+          clientSaslContext.appId,
+          clientSaslContext.saslCredentials,
+          clientSaslContext.registrationInfo)
+      } else {
+        logInfo("Add sasl client bootstrap")
+        new SaslClientBootstrap(
+          transportConf,
+          clientSaslContext.appId,
+          clientSaslContext.saslCredentials)
+      }
+    })
+    bootstrapOpt.toList.asJava
+  }
+
+  val clientFactory = transportContext.createClientFactory(createClientBootstraps())
 
   private val timeoutScheduler =
     ThreadUtils.newDaemonSingleThreadScheduledExecutor("celeborn-netty-rpc-env-timeout-checker")
@@ -92,8 +114,25 @@ class NettyRpcEnv(
     }
   }
 
+  private def createServerBootstraps(): java.util.List[TransportServerBootstrap] = {
+    val bootstrapOpt = securityContext.flatMap(_.serverSaslContext.map { serverSaslContext =>
+      if (serverSaslContext.addRegistrationBootstrap) {
+        logInfo("Add registration server bootstrap")
+        new RegistrationServerBootstrap(
+          transportConf,
+          serverSaslContext.secretRegistry)
+      } else {
+        logInfo("Add sasl server bootstrap")
+        new SaslServerBootstrap(
+          transportConf,
+          serverSaslContext.secretRegistry)
+      }
+    })
+    bootstrapOpt.toList.asJava
+  }
+
   def startServer(bindAddress: String, port: Int): Unit = {
-    server = transportContext.createServer(bindAddress, port)
+    server = transportContext.createServer(bindAddress, port, createServerBootstraps())
     dispatcher.registerRpcEndpoint(
       RpcEndpointVerifier.NAME,
       new RpcEndpointVerifier(this, dispatcher))
@@ -105,7 +144,7 @@ class NettyRpcEnv(
   }
 
   override def setupEndpoint(name: String, endpoint: RpcEndpoint): RpcEndpointRef = {
-    if (name == RpcNameConstants.WORKER_EP) {
+    if (name == RpcNameConstants.WORKER_EP || name == RpcNameConstants.WORKER_INTERNAL_EP) {
       worker = endpoint
     }
     dispatcher.registerRpcEndpoint(name, endpoint)
@@ -304,6 +343,9 @@ class NettyRpcEnv(
     if (clientFactory != null) {
       clientFactory.close()
     }
+    if (null != transportContext) {
+      transportContext.close();
+    }
     if (clientConnectionExecutor != null) {
       clientConnectionExecutor.shutdownNow()
     }
@@ -547,7 +589,7 @@ private[celeborn] class NettyRpcHandler(
     try {
       val message = requestMessage.body().nioByteBuffer()
       val messageToDispatch = internalReceive(client, message)
-      dispatcher.postRemoteMessage(messageToDispatch, callback)
+      dispatcher.postRemoteMessage(messageToDispatch, callback, client)
     } catch {
       case e: Exception =>
         val rpcReq = requestMessage.asInstanceOf[RpcRequest]
