@@ -17,8 +17,8 @@
 
 package org.apache.celeborn.common.util
 
-import java.io.{File, FileInputStream, InputStream, InputStreamReader, IOException}
-import java.lang.management.ManagementFactory
+import java.io._
+import java.lang.management.{LockInfo, ManagementFactory, MonitorInfo, ThreadInfo}
 import java.math.{MathContext, RoundingMode}
 import java.net._
 import java.nio.ByteBuffer
@@ -32,8 +32,9 @@ import scala.annotation.tailrec
 import scala.collection.JavaConverters._
 import scala.io.Source
 import scala.reflect.ClassTag
-import scala.util.Try
+import scala.util.{Random => ScalaRandom, Try}
 import scala.util.control.{ControlThrowable, NonFatal}
+import scala.util.matching.Regex
 
 import com.google.protobuf.{ByteString, GeneratedMessageV3}
 import io.netty.channel.unix.Errors.NativeIoException
@@ -45,7 +46,7 @@ import org.apache.celeborn.common.CelebornConf
 import org.apache.celeborn.common.CelebornConf.PORT_MAX_RETRY
 import org.apache.celeborn.common.exception.CelebornException
 import org.apache.celeborn.common.internal.Logging
-import org.apache.celeborn.common.meta.{DiskStatus, FileInfo, WorkerInfo}
+import org.apache.celeborn.common.meta.{DiskStatus, WorkerInfo}
 import org.apache.celeborn.common.network.protocol.TransportMessage
 import org.apache.celeborn.common.network.util.TransportConf
 import org.apache.celeborn.common.protocol.{PartitionLocation, PartitionSplitMode, PartitionType, TransportModuleConstants}
@@ -243,6 +244,10 @@ object Utils extends Logging {
     }
   }
 
+  def selectRandomPort(from: Int, to: Int): Int = {
+    ScalaRandom.nextInt(to - from) + from
+  }
+
   def startServiceOnPort[T](
       startPort: Int,
       startService: Int => (T, Int),
@@ -410,7 +415,19 @@ object Utils extends Logging {
   }
 
   def localHostName(conf: CelebornConf): String = customHostname.getOrElse {
-    if (conf.bindPreferIP) localIpAddress.getHostAddress else localIpAddress.getCanonicalHostName
+    if (conf.bindPreferIP) {
+      localIpAddress match {
+        case ipv6Address: Inet6Address =>
+          val ip = ipv6Address.getHostAddress
+          assert(
+            !ip.startsWith("[") && !ip.endsWith("]"),
+            s"Resolved IPv6 address should not be enclosed in [] but got $ip")
+          s"[$ip]"
+        case other => other.getHostAddress
+      }
+    } else {
+      localIpAddress.getCanonicalHostName
+    }
   }
 
   /**
@@ -510,10 +527,19 @@ object Utils extends Logging {
     // assuming we have all the machine's cores).
     // NB: Only set if serverThreads/clientThreads not already set.
     val numThreads = defaultNumThreads(numUsableCores)
-    conf.setIfMissing(s"celeborn.$module.io.serverThreads", numThreads.toString)
-    conf.setIfMissing(s"celeborn.$module.io.clientThreads", numThreads.toString)
+    conf.setTransportConfIfMissing(
+      module,
+      CelebornConf.NETWORK_IO_SERVER_THREADS,
+      numThreads.toString)
+    conf.setTransportConfIfMissing(
+      module,
+      CelebornConf.NETWORK_IO_CLIENT_THREADS,
+      numThreads.toString)
     if (TransportModuleConstants.PUSH_MODULE == module) {
-      conf.setIfMissing(s"celeborn.$module.io.numConnectionsPerPeer", numThreads.toString)
+      conf.setTransportConfIfMissing(
+        module,
+        CelebornConf.NETWORK_IO_NUM_CONNECTIONS_PER_PEER,
+        numThreads.toString)
     }
 
     new TransportConf(module, conf)
@@ -571,15 +597,6 @@ object Utils extends Logging {
       .orNull
   }
 
-  def getDefaultQuotaConfigurationFile(env: Map[String, String] = sys.env): String = {
-    env.get("CELEBORN_CONF_DIR")
-      .orElse(env.get("CELEBORN_HOME").map { t => s"$t${File.separator}conf" })
-      .map { t => new File(s"$t${File.separator}quota.yaml") }
-      .filter(_.isFile)
-      .map(_.getAbsolutePath)
-      .orNull
-  }
-
   private[util] def trimExceptCRLF(str: String): String = {
     val nonSpaceOrNaturalLineDelimiter: Char => Boolean = { ch =>
       ch > ' ' || ch == '\r' || ch == '\n'
@@ -663,10 +680,66 @@ object Utils extends Logging {
     System.currentTimeMillis - start
   }
 
-  def getThreadDump(): String = {
-    val runtimeMXBean = ManagementFactory.getRuntimeMXBean
-    val pid = runtimeMXBean.getName.split("@")(0)
-    runCommand(s"jstack -l ${pid}")
+  /**
+   * Note: code was initially copied from Apache Spark(v3.5.1).
+   */
+  implicit private class Lock(lock: LockInfo) {
+    def lockString: String = {
+      lock match {
+        case monitor: MonitorInfo => s"Monitor(${monitor.toString})"
+        case _ => s"Lock(${lock.toString})"
+      }
+    }
+  }
+
+  /**
+   * Return a thread dump of all threads' stacktraces.
+   *
+   * <p>Note: code was initially copied from Apache Spark(v3.5.1).
+   */
+  def getThreadDump(): Seq[ThreadStackTrace] = {
+    // We need to filter out null values here because dumpAllThreads() may return null array
+    // elements for threads that are dead / don't exist.
+    ManagementFactory.getThreadMXBean.dumpAllThreads(true, true).filter(_ != null)
+      .sortWith { case (threadTrace1, threadTrace2) =>
+        val name1 = threadTrace1.getThreadName().toLowerCase(Locale.ROOT)
+        val name2 = threadTrace2.getThreadName().toLowerCase(Locale.ROOT)
+        val nameCmpRes = name1.compareTo(name2)
+        if (nameCmpRes == 0) {
+          threadTrace1.getThreadId < threadTrace2.getThreadId
+        } else {
+          nameCmpRes < 0
+        }
+      }.map(threadInfoToThreadStackTrace)
+  }
+
+  /**
+   * Note: code was initially copied from Apache Spark(v3.5.1).
+   */
+  private def threadInfoToThreadStackTrace(threadInfo: ThreadInfo): ThreadStackTrace = {
+    val monitors = threadInfo.getLockedMonitors.map(m => m.getLockedStackFrame -> m).toMap
+    val stackTrace = StackTrace(threadInfo.getStackTrace.map { frame =>
+      monitors.get(frame) match {
+        case Some(monitor) =>
+          monitor.getLockedStackFrame.toString + s" => holding ${monitor.lockString}"
+        case None =>
+          frame.toString
+      }
+    })
+
+    // use a set to dedup re-entrant locks that are held at multiple places
+    val heldLocks =
+      (threadInfo.getLockedSynchronizers ++ threadInfo.getLockedMonitors).map(_.lockString).toSet
+
+    ThreadStackTrace(
+      threadId = threadInfo.getThreadId,
+      threadName = threadInfo.getThreadName,
+      threadState = threadInfo.getThreadState,
+      stackTrace = stackTrace,
+      blockedByThreadId =
+        if (threadInfo.getLockOwnerId < 0) None else Some(threadInfo.getLockOwnerId),
+      blockedByLock = Option(threadInfo.getLockInfo).map(_.lockString).getOrElse(""),
+      holdingLocks = heldLocks.toSeq)
   }
 
   private def readProcessStdout(process: Process): String = {
@@ -945,7 +1018,7 @@ object Utils extends Logging {
       case 16 =>
         StatusCode.STAGE_ENDED
       case 17 =>
-        StatusCode.PUSH_DATA_FAIL_NON_CRITICAL_CAUSE
+        StatusCode.PUSH_DATA_FAIL_NON_CRITICAL_CAUSE_PRIMARY
       case 18 =>
         StatusCode.PUSH_DATA_WRITE_FAIL_REPLICA
       case 19 =>
@@ -996,6 +1069,8 @@ object Utils extends Logging {
         StatusCode.DESTROY_SLOTS_MOCK_FAILURE
       case 49 =>
         StatusCode.COMMIT_FILES_MOCK_FAILURE
+      case 50 =>
+        StatusCode.PUSH_DATA_FAIL_NON_CRITICAL_CAUSE_REPLICA
       case _ =>
         null
     }
@@ -1105,13 +1180,6 @@ object Utils extends Logging {
     }
   }
 
-  def getShortFormattedFileName(fileInfo: FileInfo): String = {
-    val parentFile = fileInfo.getFile.getParent
-    parentFile.substring(
-      parentFile.lastIndexOf("/"),
-      parentFile.length) + "/" + fileInfo.getFile.getName
-  }
-
   def parseMetricLabels(label: String): (String, String) = {
     val labelPart = label.split("=")
     if (labelPart.size != 2) {
@@ -1125,4 +1193,51 @@ object Utils extends Logging {
   private val dateFmt: FastDateFormat =
     FastDateFormat.getInstance("yyyy-MM-dd'T'HH:mm:ss.SSSZ", Locale.ROOT)
   def formatTimestamp(timestamp: Long): String = dateFmt.format(timestamp)
+
+  def parseColonSeparatedHostPorts(id: String, portsNum: Int): Array[String] = {
+    val components = id.split(":")
+    val portsArr = components.takeRight(portsNum)
+    val host = components.dropRight(portsNum).mkString(":")
+    Array(host) ++ portsArr
+  }
+
+  private val REDACTION_REPLACEMENT_TEXT = "*********(redacted)"
+
+  /**
+   * Redact the sensitive values in the given map. If a map key matches the redaction pattern then
+   * its value is replaced with a dummy text.
+   */
+  def redact(conf: CelebornConf, kvs: Seq[(String, String)]): Seq[(String, String)] = {
+    val redactionPattern = conf.secretRedactionPattern
+    redact(redactionPattern, kvs)
+  }
+
+  private def redact[K, V](redactionPattern: Regex, kvs: Seq[(K, V)]): Seq[(K, V)] = {
+    // If the sensitive information regex matches with either the key or the value, redact the value
+    // While the original intent was to only redact the value if the key matched with the regex,
+    // we've found that especially in verbose mode, the value of the property may contain sensitive
+    // information like so:
+    //
+    // celeborn.dynamicConfig.store.db.hikari.password=secret_password ...
+    //
+    // And, in such cases, simply searching for the sensitive information regex in the key name is
+    // not sufficient. The values themselves have to be searched as well and redacted if matched.
+    // This does mean we may be accounting more false positives - for example, if the value of an
+    // arbitrary property contained the term 'password', we may redact the value from the UI and
+    // logs. In order to work around it, user would have to make the celeborn.redaction.regex property
+    // more specific.
+    kvs.map {
+      case (key: String, value: String) =>
+        redactionPattern.findFirstIn(key)
+          .orElse(redactionPattern.findFirstIn(value))
+          .map { _ => (key, REDACTION_REPLACEMENT_TEXT) }
+          .getOrElse((key, value))
+      case (key, value: String) =>
+        redactionPattern.findFirstIn(value)
+          .map { _ => (key, REDACTION_REPLACEMENT_TEXT) }
+          .getOrElse((key, value))
+      case (key, value) =>
+        (key, value)
+    }.asInstanceOf[Seq[(K, V)]]
+  }
 }
