@@ -27,6 +27,7 @@ import java.util.concurrent.atomic.AtomicLong;
 
 import scala.Tuple2;
 
+import com.google.common.collect.Lists;
 import org.junit.*;
 import org.mockito.Mockito;
 
@@ -36,13 +37,17 @@ import org.apache.celeborn.common.exception.CelebornRuntimeException;
 import org.apache.celeborn.common.identity.UserIdentifier;
 import org.apache.celeborn.common.meta.DiskInfo;
 import org.apache.celeborn.common.meta.WorkerInfo;
+import org.apache.celeborn.common.meta.WorkerStatus;
+import org.apache.celeborn.common.protocol.WorkerEventType;
 import org.apache.celeborn.common.quota.ResourceConsumption;
 import org.apache.celeborn.common.rpc.RpcEndpointAddress;
 import org.apache.celeborn.common.rpc.RpcEndpointRef;
 import org.apache.celeborn.common.rpc.RpcEnv;
 import org.apache.celeborn.common.rpc.netty.NettyRpcEndpointRef;
 import org.apache.celeborn.common.util.Utils;
+import org.apache.celeborn.common.util.Utils$;
 import org.apache.celeborn.service.deploy.master.clustermeta.AbstractMetaManager;
+import org.apache.celeborn.service.deploy.master.clustermeta.ResourceProtos;
 
 public class RatisMasterStatusSystemSuiteJ {
   protected static HARaftServer RATISSERVER1 = null;
@@ -60,99 +65,120 @@ public class RatisMasterStatusSystemSuiteJ {
   protected static RpcEndpointRef mockRpcEndpoint = Mockito.mock(RpcEndpointRef.class);
 
   @BeforeClass
-  public static void init() throws IOException, InterruptedException {
-    resetRaftServer();
+  public static void init() throws Exception {
+    resetRaftServer(
+        configureServerConf(new CelebornConf(), 1),
+        configureServerConf(new CelebornConf(), 2),
+        configureServerConf(new CelebornConf(), 3),
+        false);
   }
 
-  public static void resetRaftServer() throws IOException, InterruptedException {
+  private static void stopAllRaftServers() {
+    if (RATISSERVER1 != null) {
+      RATISSERVER1.stop();
+    }
+    if (RATISSERVER2 != null) {
+      RATISSERVER2.stop();
+    }
+    if (RATISSERVER3 != null) {
+      RATISSERVER3.stop();
+    }
+  }
+
+  static CelebornConf configureServerConf(CelebornConf conf, int id) throws IOException {
+    File tmpDir = File.createTempFile("celeborn-ratis" + id, "for-test-only");
+    tmpDir.delete();
+    tmpDir.mkdirs();
+    conf.set(CelebornConf.HA_MASTER_RATIS_STORAGE_DIR().key(), tmpDir.getAbsolutePath());
+    return conf;
+  }
+
+  public static void resetRaftServer(
+      CelebornConf conf1, CelebornConf conf2, CelebornConf conf3, boolean sslEnabled)
+      throws IOException, InterruptedException {
     Mockito.when(mockRpcEnv.setupEndpointRef(Mockito.any(), Mockito.any()))
         .thenReturn(mockRpcEndpoint);
     when(mockRpcEnv.setupEndpointRef(any(), any())).thenReturn(dummyRef);
 
-    if (RATISSERVER1 != null) {
-      RATISSERVER1.stop();
+    stopAllRaftServers();
+
+    int retryCount = 0;
+    boolean serversStarted = false;
+
+    while (!serversStarted) {
+      try {
+        STATUSSYSTEM1 = new HAMasterMetaManager(mockRpcEnv, new CelebornConf());
+        STATUSSYSTEM2 = new HAMasterMetaManager(mockRpcEnv, new CelebornConf());
+        STATUSSYSTEM3 = new HAMasterMetaManager(mockRpcEnv, new CelebornConf());
+
+        MetaHandler handler1 = new MetaHandler(STATUSSYSTEM1);
+        MetaHandler handler2 = new MetaHandler(STATUSSYSTEM2);
+        MetaHandler handler3 = new MetaHandler(STATUSSYSTEM3);
+
+        String id1 = UUID.randomUUID().toString();
+        String id2 = UUID.randomUUID().toString();
+        String id3 = UUID.randomUUID().toString();
+
+        int ratisPort1 = Utils$.MODULE$.selectRandomPort(1024, 65535);
+        int ratisPort2 = ratisPort1 + 1;
+        int ratisPort3 = ratisPort2 + 1;
+
+        MasterNode masterNode1 =
+            new MasterNode.Builder()
+                .setHost(Utils.localHostName(conf1))
+                .setRatisPort(ratisPort1)
+                .setRpcPort(ratisPort1)
+                .setInternalRpcPort(ratisPort1)
+                .setSslEnabled(sslEnabled)
+                .setNodeId(id1)
+                .build();
+        MasterNode masterNode2 =
+            new MasterNode.Builder()
+                .setHost(Utils.localHostName(conf2))
+                .setRatisPort(ratisPort2)
+                .setRpcPort(ratisPort2)
+                .setInternalRpcPort(ratisPort2)
+                .setSslEnabled(sslEnabled)
+                .setNodeId(id2)
+                .build();
+        MasterNode masterNode3 =
+            new MasterNode.Builder()
+                .setHost(Utils.localHostName(conf3))
+                .setRatisPort(ratisPort3)
+                .setRpcPort(ratisPort3)
+                .setInternalRpcPort(ratisPort3)
+                .setSslEnabled(sslEnabled)
+                .setNodeId(id3)
+                .build();
+
+        List<MasterNode> peersForNode1 = Arrays.asList(masterNode2, masterNode3);
+        List<MasterNode> peersForNode2 = Arrays.asList(masterNode1, masterNode3);
+        List<MasterNode> peersForNode3 = Arrays.asList(masterNode1, masterNode2);
+
+        RATISSERVER1 =
+            HARaftServer.newMasterRatisServer(handler1, conf1, masterNode1, peersForNode1);
+        RATISSERVER2 =
+            HARaftServer.newMasterRatisServer(handler2, conf2, masterNode2, peersForNode2);
+        RATISSERVER3 =
+            HARaftServer.newMasterRatisServer(handler3, conf3, masterNode3, peersForNode3);
+
+        STATUSSYSTEM1.setRatisServer(RATISSERVER1);
+        STATUSSYSTEM2.setRatisServer(RATISSERVER2);
+        STATUSSYSTEM3.setRatisServer(RATISSERVER3);
+
+        RATISSERVER1.start();
+        RATISSERVER2.start();
+        RATISSERVER3.start();
+        Thread.sleep(15 * 1000);
+        serversStarted = true;
+      } catch (Exception e) {
+        stopAllRaftServers();
+        retryCount += 1;
+        if (retryCount == 3) {
+          throw e;
+        }
+      }
     }
-
-    if (RATISSERVER2 != null) {
-      RATISSERVER2.stop();
-    }
-
-    if (RATISSERVER3 != null) {
-      RATISSERVER3.stop();
-    }
-
-    STATUSSYSTEM1 = new HAMasterMetaManager(mockRpcEnv, new CelebornConf());
-    STATUSSYSTEM2 = new HAMasterMetaManager(mockRpcEnv, new CelebornConf());
-    STATUSSYSTEM3 = new HAMasterMetaManager(mockRpcEnv, new CelebornConf());
-
-    MetaHandler handler1 = new MetaHandler(STATUSSYSTEM1);
-    MetaHandler handler2 = new MetaHandler(STATUSSYSTEM2);
-    MetaHandler handler3 = new MetaHandler(STATUSSYSTEM3);
-
-    CelebornConf conf1 = new CelebornConf();
-    File tmpDir1 = File.createTempFile("celeborn-ratis1", "for-test-only");
-    tmpDir1.delete();
-    tmpDir1.mkdirs();
-    conf1.set(CelebornConf.HA_MASTER_RATIS_STORAGE_DIR().key(), tmpDir1.getAbsolutePath());
-
-    CelebornConf conf2 = new CelebornConf();
-    File tmpDir2 = File.createTempFile("celeborn-ratis2", "for-test-only");
-    tmpDir2.delete();
-    tmpDir2.mkdirs();
-    conf2.set(CelebornConf.HA_MASTER_RATIS_STORAGE_DIR().key(), tmpDir2.getAbsolutePath());
-
-    CelebornConf conf3 = new CelebornConf();
-    File tmpDir3 = File.createTempFile("celeborn-ratis3", "for-test-only");
-    tmpDir3.delete();
-    tmpDir3.mkdirs();
-    conf3.set(CelebornConf.HA_MASTER_RATIS_STORAGE_DIR().key(), tmpDir3.getAbsolutePath());
-
-    String id1 = UUID.randomUUID().toString();
-    String id2 = UUID.randomUUID().toString();
-    String id3 = UUID.randomUUID().toString();
-    int ratisPort1 = 9872;
-    int ratisPort2 = 9873;
-    int ratisPort3 = 9874;
-
-    MasterNode masterNode1 =
-        new MasterNode.Builder()
-            .setHost(Utils.localHostName(conf1))
-            .setRatisPort(ratisPort1)
-            .setRpcPort(9872)
-            .setNodeId(id1)
-            .build();
-    MasterNode masterNode2 =
-        new MasterNode.Builder()
-            .setHost(Utils.localHostName(conf2))
-            .setRatisPort(ratisPort2)
-            .setRpcPort(9873)
-            .setNodeId(id2)
-            .build();
-    MasterNode masterNode3 =
-        new MasterNode.Builder()
-            .setHost(Utils.localHostName(conf3))
-            .setRatisPort(ratisPort3)
-            .setRpcPort(9874)
-            .setNodeId(id3)
-            .build();
-
-    List<MasterNode> peersForNode1 = Arrays.asList(masterNode2, masterNode3);
-    List<MasterNode> peersForNode2 = Arrays.asList(masterNode1, masterNode3);
-    List<MasterNode> peersForNode3 = Arrays.asList(masterNode1, masterNode2);
-
-    RATISSERVER1 = HARaftServer.newMasterRatisServer(handler1, conf1, masterNode1, peersForNode1);
-    RATISSERVER2 = HARaftServer.newMasterRatisServer(handler2, conf2, masterNode2, peersForNode2);
-    RATISSERVER3 = HARaftServer.newMasterRatisServer(handler3, conf3, masterNode3, peersForNode3);
-
-    STATUSSYSTEM1.setRatisServer(RATISSERVER1);
-    STATUSSYSTEM2.setRatisServer(RATISSERVER2);
-    STATUSSYSTEM3.setRatisServer(RATISSERVER3);
-
-    RATISSERVER1.start();
-    RATISSERVER2.start();
-    RATISSERVER3.start();
-
-    Thread.sleep(15 * 1000);
   }
 
   @Test
@@ -161,7 +187,7 @@ public class RatisMasterStatusSystemSuiteJ {
         RATISSERVER1.isLeader() || RATISSERVER2.isLeader() || RATISSERVER3.isLeader();
     Assert.assertTrue(hasLeader);
 
-    // Check if the rpc endpoint of the leader is as expected.
+    // Check if the rpc endpoint and internal rpc endpoint of the leader is as expected.
 
     HARaftServer leader =
         RATISSERVER1.isLeader()
@@ -180,11 +206,16 @@ public class RatisMasterStatusSystemSuiteJ {
     Assert.assertTrue(cachedLeaderPeerRpcEndpoint.isPresent());
 
     Tuple2<String, String> rpcEndpointsPair = cachedLeaderPeerRpcEndpoint.get().rpcEndpoints;
+    Tuple2<String, String> rpcInternalEndpointsPair =
+        cachedLeaderPeerRpcEndpoint.get().rpcInternalEndpoints;
 
     // rpc endpoint may use custom host name then this ut need check ever ip/host
     Assert.assertTrue(
         leader.getRpcEndpoint().equals(rpcEndpointsPair._1)
             || leader.getRpcEndpoint().equals(rpcEndpointsPair._2));
+    Assert.assertTrue(
+        leader.getInternalRpcEndpoint().equals(rpcInternalEndpointsPair._1)
+            || leader.getRpcEndpoint().equals(rpcInternalEndpointsPair._2));
   }
 
   private static final String HOSTNAME1 = "host1";
@@ -192,6 +223,8 @@ public class RatisMasterStatusSystemSuiteJ {
   private static final int PUSHPORT1 = 1112;
   private static final int FETCHPORT1 = 1113;
   private static final int REPLICATEPORT1 = 1114;
+  private static final int INTERNALPORT1 = 1115;
+  private static final String NETWORK_LOCATION1 = "networkLocation1";
   private static final Map<String, DiskInfo> disks1 = new HashMap<>();
   private static final Map<UserIdentifier, ResourceConsumption> userResourceConsumption1 =
       new HashMap<>();
@@ -201,6 +234,8 @@ public class RatisMasterStatusSystemSuiteJ {
   private static final int PUSHPORT2 = 2112;
   private static final int FETCHPORT2 = 2113;
   private static final int REPLICATEPORT2 = 2114;
+  private static final int INTERNALPORT2 = 2115;
+  private static final String NETWORK_LOCATION2 = "networkLocation2";
   private static final Map<String, DiskInfo> disks2 = new HashMap<>();
   private static final Map<UserIdentifier, ResourceConsumption> userResourceConsumption2 =
       new HashMap<>();
@@ -210,6 +245,8 @@ public class RatisMasterStatusSystemSuiteJ {
   private static final int PUSHPORT3 = 3112;
   private static final int FETCHPORT3 = 3113;
   private static final int REPLICATEPORT3 = 3114;
+  private static final int INTERNALPORT3 = 3115;
+  private static final String NETWORK_LOCATION3 = "networkLocation3";
   private static final Map<String, DiskInfo> disks3 = new HashMap<>();
   private static final Map<UserIdentifier, ResourceConsumption> userResourceConsumption3 =
       new HashMap<>();
@@ -218,6 +255,7 @@ public class RatisMasterStatusSystemSuiteJ {
   private static final String APPID1 = "appId1";
   private static final int SHUFFLEID1 = 1;
   private static final String SHUFFLEKEY1 = APPID1 + "-" + SHUFFLEID1;
+  private static final WorkerStatus workerStatus = WorkerStatus.normalWorkerStatus();
 
   private String getNewReqeustId() {
     return MasterClient.encodeRequestId(UUID.randomUUID().toString(), callerId.incrementAndGet());
@@ -256,6 +294,8 @@ public class RatisMasterStatusSystemSuiteJ {
           PUSHPORT1,
           FETCHPORT1,
           REPLICATEPORT1,
+          INTERNALPORT1,
+          NETWORK_LOCATION1,
           disks1,
           userResourceConsumption1,
           getNewReqeustId());
@@ -263,7 +303,7 @@ public class RatisMasterStatusSystemSuiteJ {
     } catch (CelebornRuntimeException e) {
       Assert.assertTrue(true);
     } finally {
-      resetRaftServer();
+      init();
     }
   }
 
@@ -278,6 +318,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT1,
         FETCHPORT1,
         REPLICATEPORT1,
+        INTERNALPORT1,
+        NETWORK_LOCATION1,
         disks1,
         userResourceConsumption1,
         getNewReqeustId());
@@ -287,6 +329,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT2,
         FETCHPORT2,
         REPLICATEPORT2,
+        INTERNALPORT2,
+        NETWORK_LOCATION2,
         disks2,
         userResourceConsumption2,
         getNewReqeustId());
@@ -296,6 +340,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT3,
         FETCHPORT3,
         REPLICATEPORT3,
+        INTERNALPORT3,
+        NETWORK_LOCATION3,
         disks3,
         userResourceConsumption3,
         getNewReqeustId());
@@ -319,6 +365,7 @@ public class RatisMasterStatusSystemSuiteJ {
             PUSHPORT1,
             FETCHPORT1,
             REPLICATEPORT1,
+            INTERNALPORT1,
             disks1,
             userResourceConsumption1);
     WorkerInfo workerInfo2 =
@@ -328,6 +375,7 @@ public class RatisMasterStatusSystemSuiteJ {
             PUSHPORT2,
             FETCHPORT2,
             REPLICATEPORT2,
+            INTERNALPORT2,
             disks2,
             userResourceConsumption2);
 
@@ -337,6 +385,8 @@ public class RatisMasterStatusSystemSuiteJ {
         workerInfo1.pushPort(),
         workerInfo1.fetchPort(),
         workerInfo1.replicatePort(),
+        workerInfo1.internalPort(),
+        workerInfo1.networkLocation(),
         workerInfo1.diskInfos(),
         workerInfo1.userResourceConsumption(),
         getNewReqeustId());
@@ -346,6 +396,8 @@ public class RatisMasterStatusSystemSuiteJ {
         workerInfo2.pushPort(),
         workerInfo2.fetchPort(),
         workerInfo2.replicatePort(),
+        workerInfo2.internalPort(),
+        workerInfo2.networkLocation(),
         workerInfo2.diskInfos(),
         workerInfo2.userResourceConsumption(),
         getNewReqeustId());
@@ -378,6 +430,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT1,
         FETCHPORT1,
         REPLICATEPORT1,
+        INTERNALPORT1,
+        NETWORK_LOCATION1,
         disks1,
         userResourceConsumption1,
         getNewReqeustId());
@@ -387,6 +441,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT2,
         FETCHPORT2,
         REPLICATEPORT2,
+        INTERNALPORT2,
+        NETWORK_LOCATION2,
         disks2,
         userResourceConsumption2,
         getNewReqeustId());
@@ -396,6 +452,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT3,
         FETCHPORT3,
         REPLICATEPORT3,
+        INTERNALPORT3,
+        NETWORK_LOCATION3,
         disks3,
         userResourceConsumption3,
         getNewReqeustId());
@@ -420,6 +478,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT1,
         FETCHPORT1,
         REPLICATEPORT1,
+        INTERNALPORT1,
+        NETWORK_LOCATION1,
         disks1,
         userResourceConsumption1,
         getNewReqeustId());
@@ -429,6 +489,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT2,
         FETCHPORT2,
         REPLICATEPORT2,
+        INTERNALPORT2,
+        NETWORK_LOCATION2,
         disks2,
         userResourceConsumption2,
         getNewReqeustId());
@@ -438,6 +500,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT3,
         FETCHPORT3,
         REPLICATEPORT3,
+        INTERNALPORT3,
+        NETWORK_LOCATION3,
         disks3,
         userResourceConsumption3,
         getNewReqeustId());
@@ -449,6 +513,7 @@ public class RatisMasterStatusSystemSuiteJ {
             PUSHPORT1,
             FETCHPORT1,
             REPLICATEPORT1,
+            INTERNALPORT1,
             disks1,
             userResourceConsumption1);
     WorkerInfo workerInfo2 =
@@ -458,6 +523,7 @@ public class RatisMasterStatusSystemSuiteJ {
             PUSHPORT2,
             FETCHPORT2,
             REPLICATEPORT2,
+            INTERNALPORT2,
             disks2,
             userResourceConsumption2);
     WorkerInfo workerInfo3 =
@@ -467,6 +533,7 @@ public class RatisMasterStatusSystemSuiteJ {
             PUSHPORT3,
             FETCHPORT3,
             REPLICATEPORT3,
+            INTERNALPORT3,
             disks3,
             userResourceConsumption3);
 
@@ -519,6 +586,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT1,
         FETCHPORT1,
         REPLICATEPORT1,
+        INTERNALPORT1,
+        NETWORK_LOCATION1,
         disks1,
         userResourceConsumption1,
         getNewReqeustId());
@@ -528,6 +597,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT2,
         FETCHPORT2,
         REPLICATEPORT2,
+        INTERNALPORT2,
+        NETWORK_LOCATION3,
         disks2,
         userResourceConsumption2,
         getNewReqeustId());
@@ -537,6 +608,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT3,
         FETCHPORT3,
         REPLICATEPORT3,
+        INTERNALPORT3,
+        NETWORK_LOCATION3,
         disks3,
         userResourceConsumption3,
         getNewReqeustId());
@@ -603,6 +676,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT1,
         FETCHPORT1,
         REPLICATEPORT1,
+        INTERNALPORT1,
+        NETWORK_LOCATION1,
         disks1,
         userResourceConsumption1,
         getNewReqeustId());
@@ -612,6 +687,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT2,
         FETCHPORT2,
         REPLICATEPORT2,
+        INTERNALPORT2,
+        NETWORK_LOCATION2,
         disks2,
         userResourceConsumption2,
         getNewReqeustId());
@@ -621,6 +698,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT3,
         FETCHPORT3,
         REPLICATEPORT3,
+        INTERNALPORT3,
+        NETWORK_LOCATION3,
         disks3,
         userResourceConsumption3,
         getNewReqeustId());
@@ -633,6 +712,7 @@ public class RatisMasterStatusSystemSuiteJ {
             PUSHPORT1,
             FETCHPORT1,
             REPLICATEPORT1,
+            INTERNALPORT1,
             disks1,
             userResourceConsumption1);
     WorkerInfo workerInfo2 =
@@ -642,6 +722,7 @@ public class RatisMasterStatusSystemSuiteJ {
             PUSHPORT2,
             FETCHPORT2,
             REPLICATEPORT2,
+            INTERNALPORT2,
             disks2,
             userResourceConsumption2);
     Map<String, Map<String, Integer>> workersToAllocate = new HashMap<>();
@@ -676,6 +757,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT1,
         FETCHPORT1,
         REPLICATEPORT1,
+        INTERNALPORT1,
+        NETWORK_LOCATION1,
         disks1,
         userResourceConsumption1,
         getNewReqeustId());
@@ -685,6 +768,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT2,
         FETCHPORT2,
         REPLICATEPORT2,
+        INTERNALPORT2,
+        NETWORK_LOCATION2,
         disks2,
         userResourceConsumption2,
         getNewReqeustId());
@@ -694,6 +779,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT3,
         FETCHPORT3,
         REPLICATEPORT3,
+        INTERNALPORT3,
+        NETWORK_LOCATION3,
         disks3,
         userResourceConsumption3,
         getNewReqeustId());
@@ -705,6 +792,7 @@ public class RatisMasterStatusSystemSuiteJ {
             PUSHPORT1,
             FETCHPORT1,
             REPLICATEPORT1,
+            INTERNALPORT1,
             disks1,
             userResourceConsumption1);
     WorkerInfo workerInfo2 =
@@ -714,6 +802,7 @@ public class RatisMasterStatusSystemSuiteJ {
             PUSHPORT2,
             FETCHPORT2,
             REPLICATEPORT2,
+            INTERNALPORT2,
             disks2,
             userResourceConsumption2);
 
@@ -774,6 +863,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT1,
         FETCHPORT1,
         REPLICATEPORT1,
+        INTERNALPORT1,
+        NETWORK_LOCATION1,
         disks1,
         userResourceConsumption1,
         getNewReqeustId());
@@ -783,6 +874,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT2,
         FETCHPORT2,
         REPLICATEPORT2,
+        INTERNALPORT2,
+        NETWORK_LOCATION2,
         disks2,
         userResourceConsumption2,
         getNewReqeustId());
@@ -792,6 +885,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT3,
         FETCHPORT3,
         REPLICATEPORT3,
+        INTERNALPORT3,
+        NETWORK_LOCATION3,
         disks3,
         userResourceConsumption3,
         getNewReqeustId());
@@ -807,6 +902,7 @@ public class RatisMasterStatusSystemSuiteJ {
         new HashMap<>(),
         1,
         false,
+        workerStatus,
         getNewReqeustId());
     Thread.sleep(3000L);
 
@@ -825,6 +921,7 @@ public class RatisMasterStatusSystemSuiteJ {
         new HashMap<>(),
         1,
         false,
+        workerStatus,
         getNewReqeustId());
     Thread.sleep(3000L);
 
@@ -844,6 +941,7 @@ public class RatisMasterStatusSystemSuiteJ {
         new HashMap<>(),
         1,
         false,
+        workerStatus,
         getNewReqeustId());
     Thread.sleep(3000L);
 
@@ -863,6 +961,7 @@ public class RatisMasterStatusSystemSuiteJ {
         new HashMap<>(),
         1,
         true,
+        workerStatus,
         getNewReqeustId());
     Thread.sleep(3000L);
     Assert.assertEquals(2, statusSystem.excludedWorkers.size());
@@ -924,6 +1023,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT1,
         FETCHPORT1,
         REPLICATEPORT1,
+        INTERNALPORT1,
+        NETWORK_LOCATION1,
         disks1,
         userResourceConsumption1,
         getNewReqeustId());
@@ -933,6 +1034,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT2,
         FETCHPORT2,
         REPLICATEPORT2,
+        INTERNALPORT2,
+        NETWORK_LOCATION2,
         disks2,
         userResourceConsumption2,
         getNewReqeustId());
@@ -942,6 +1045,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT3,
         FETCHPORT3,
         REPLICATEPORT3,
+        INTERNALPORT3,
+        NETWORK_LOCATION3,
         disks3,
         userResourceConsumption3,
         getNewReqeustId());
@@ -954,6 +1059,7 @@ public class RatisMasterStatusSystemSuiteJ {
             PUSHPORT1,
             FETCHPORT1,
             REPLICATEPORT1,
+            INTERNALPORT1,
             disks1,
             userResourceConsumption1));
 
@@ -978,6 +1084,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT1,
         FETCHPORT1,
         REPLICATEPORT1,
+        INTERNALPORT1,
+        NETWORK_LOCATION1,
         disks1,
         userResourceConsumption1,
         getNewReqeustId());
@@ -987,6 +1095,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT2,
         FETCHPORT2,
         REPLICATEPORT2,
+        INTERNALPORT2,
+        NETWORK_LOCATION2,
         disks2,
         userResourceConsumption2,
         getNewReqeustId());
@@ -996,6 +1106,8 @@ public class RatisMasterStatusSystemSuiteJ {
         PUSHPORT3,
         FETCHPORT3,
         REPLICATEPORT3,
+        INTERNALPORT3,
+        NETWORK_LOCATION3,
         disks3,
         userResourceConsumption3,
         getNewReqeustId());
@@ -1007,6 +1119,7 @@ public class RatisMasterStatusSystemSuiteJ {
             PUSHPORT1,
             FETCHPORT1,
             REPLICATEPORT1,
+            INTERNALPORT1,
             disks1,
             userResourceConsumption1);
 
@@ -1076,6 +1189,82 @@ public class RatisMasterStatusSystemSuiteJ {
     Assert.assertEquals(statusSystem.estimatedPartitionSize, conf.minPartitionSizeToEstimate());
   }
 
+  @Test
+  public void testHandleWorkerEvent() throws InterruptedException {
+    AbstractMetaManager statusSystem = pickLeaderStatusSystem();
+    Assert.assertNotNull(statusSystem);
+
+    statusSystem.handleRegisterWorker(
+        HOSTNAME1,
+        RPCPORT1,
+        PUSHPORT1,
+        FETCHPORT1,
+        REPLICATEPORT1,
+        INTERNALPORT1,
+        NETWORK_LOCATION1,
+        disks1,
+        userResourceConsumption1,
+        getNewReqeustId());
+    statusSystem.handleRegisterWorker(
+        HOSTNAME2,
+        RPCPORT2,
+        PUSHPORT2,
+        FETCHPORT2,
+        REPLICATEPORT2,
+        INTERNALPORT2,
+        NETWORK_LOCATION2,
+        disks2,
+        userResourceConsumption2,
+        getNewReqeustId());
+    statusSystem.handleRegisterWorker(
+        HOSTNAME3,
+        RPCPORT3,
+        PUSHPORT3,
+        FETCHPORT3,
+        REPLICATEPORT3,
+        INTERNALPORT3,
+        NETWORK_LOCATION3,
+        disks3,
+        userResourceConsumption3,
+        getNewReqeustId());
+
+    WorkerInfo workerInfo1 =
+        WorkerInfo.fromUniqueId(
+            HOSTNAME1 + ":" + RPCPORT1 + ":" + PUSHPORT1 + ":" + FETCHPORT1 + ":" + REPLICATEPORT1);
+    WorkerInfo workerInfo2 =
+        WorkerInfo.fromUniqueId(
+            HOSTNAME2 + ":" + RPCPORT2 + ":" + PUSHPORT2 + ":" + FETCHPORT2 + ":" + REPLICATEPORT2);
+    statusSystem.handleWorkerEvent(
+        ResourceProtos.WorkerEventType.Decommission_VALUE,
+        Lists.newArrayList(workerInfo1, workerInfo2),
+        getNewReqeustId());
+
+    Thread.sleep(3000L);
+    Assert.assertEquals(2, STATUSSYSTEM1.workerEventInfos.size());
+    Assert.assertEquals(2, STATUSSYSTEM2.workerEventInfos.size());
+    Assert.assertEquals(2, STATUSSYSTEM3.workerEventInfos.size());
+
+    Assert.assertTrue(STATUSSYSTEM1.workerEventInfos.containsKey(workerInfo1));
+    Assert.assertTrue(STATUSSYSTEM1.workerEventInfos.containsKey(workerInfo2));
+
+    Assert.assertEquals(
+        WorkerEventType.Decommission,
+        STATUSSYSTEM1.workerEventInfos.get(workerInfo1).getEventType());
+
+    statusSystem.handleWorkerEvent(
+        ResourceProtos.WorkerEventType.None_VALUE,
+        Lists.newArrayList(workerInfo1),
+        getNewReqeustId());
+    Thread.sleep(3000L);
+    Assert.assertEquals(1, STATUSSYSTEM1.workerEventInfos.size());
+    Assert.assertEquals(1, STATUSSYSTEM2.workerEventInfos.size());
+    Assert.assertEquals(1, STATUSSYSTEM3.workerEventInfos.size());
+    Assert.assertTrue(STATUSSYSTEM1.workerEventInfos.containsKey(workerInfo2));
+    Assert.assertEquals(
+        WorkerEventType.Decommission,
+        STATUSSYSTEM1.workerEventInfos.get(workerInfo2).getEventType());
+  }
+
   @AfterClass
   public static void testNotifyLogFailed() {
     List<HARaftServer> list = Arrays.asList(RATISSERVER1, RATISSERVER2, RATISSERVER3);
