@@ -21,7 +21,6 @@ import java.io.File;
 import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.nio.channels.FileChannel;
-import java.util.Optional;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
@@ -50,7 +49,9 @@ import org.apache.celeborn.common.protocol.StorageInfo;
 import org.apache.celeborn.common.unsafe.Platform;
 import org.apache.celeborn.common.util.FileChannelUtils;
 import org.apache.celeborn.service.deploy.worker.WorkerSource;
+import org.apache.celeborn.service.deploy.worker.congestcontrol.BufferStatusHub;
 import org.apache.celeborn.service.deploy.worker.congestcontrol.CongestionController;
+import org.apache.celeborn.service.deploy.worker.congestcontrol.UserBufferInfo;
 import org.apache.celeborn.service.deploy.worker.memory.MemoryManager;
 
 /*
@@ -102,6 +103,9 @@ public abstract class PartitionDataWriter implements DeviceObserver {
   private final long hdfsFlusherBufferSize;
   private Exception exception = null;
   private boolean metricsCollectCriticalEnabled;
+  private long chunkSize;
+
+  private UserBufferInfo userBufferInfo = null;
 
   public PartitionDataWriter(
       StorageManager storageManager,
@@ -125,6 +129,7 @@ public abstract class PartitionDataWriter implements DeviceObserver {
     this.localFlusherBufferSize = conf.workerFlusherBufferSize();
     this.hdfsFlusherBufferSize = conf.workerHdfsFlusherBufferSize();
     this.metricsCollectCriticalEnabled = conf.metricsCollectCriticalEnabled();
+    this.chunkSize = conf.shuffleChunkSize();
 
     Tuple4<MemoryFileInfo, Flusher, DiskFileInfo, File> createFileResult =
         storageManager.createFile(writerContext, supportInMemory);
@@ -138,6 +143,7 @@ public abstract class PartitionDataWriter implements DeviceObserver {
     } else if (createFileResult._2() != null) {
       this.diskFileInfo = createFileResult._3();
       this.flusher = createFileResult._2();
+      this.flushWorkerIndex = this.flusher.getWorkerIndex();
       File workingDir = createFileResult._4();
       this.isMemoryShuffleFile.set(false);
       initFileChannelsForDiskFile();
@@ -153,6 +159,10 @@ public abstract class PartitionDataWriter implements DeviceObserver {
       this.mapIdBitMap = new RoaringBitmap();
     }
     takeBuffer();
+    CongestionController congestionController = CongestionController.instance();
+    if (!isMemoryShuffleFile.get() && congestionController != null) {
+      userBufferInfo = congestionController.getUserBuffer(getDiskFileInfo().getUserIdentifier());
+    }
   }
 
   public void initFileChannelsForDiskFile() throws IOException {
@@ -216,16 +226,20 @@ public abstract class PartitionDataWriter implements DeviceObserver {
           MemoryManager.instance().incrementDiskBuffer(numBytes);
           // read flush buffer to generate correct chunk offsets
           // data header layout (mapId, attemptId, nextBatchId, length)
-          ByteBuffer headerBuf = ByteBuffer.allocate(16);
-          while (dupBuf.isReadable()) {
-            headerBuf.rewind();
-            dupBuf.readBytes(headerBuf);
-            byte[] batchHeader = headerBuf.array();
-            int compressedSize = Platform.getInt(batchHeader, Platform.BYTE_ARRAY_OFFSET + 12);
-            dupBuf.skipBytes(compressedSize);
-            diskFileInfo.updateBytesFlushed(compressedSize + 16);
+          if (numBytes > chunkSize) {
+            ByteBuffer headerBuf = ByteBuffer.allocate(16);
+            while (dupBuf.isReadable()) {
+              headerBuf.rewind();
+              dupBuf.readBytes(headerBuf);
+              byte[] batchHeader = headerBuf.array();
+              int compressedSize = Platform.getInt(batchHeader, Platform.BYTE_ARRAY_OFFSET + 12);
+              dupBuf.skipBytes(compressedSize);
+              diskFileInfo.updateBytesFlushed(compressedSize + 16);
+            }
+            dupBuf.release();
+          } else {
+            diskFileInfo.updateBytesFlushed(numBytes);
           }
-          dupBuf.release();
         } else {
           if (!isMemoryShuffleFile.get()) {
             notifier.numPendingFlushes.incrementAndGet();
@@ -288,10 +302,10 @@ public abstract class PartitionDataWriter implements DeviceObserver {
       MemoryManager.instance().increaseMemoryFileStorage(numBytes);
     } else {
       MemoryManager.instance().incrementDiskBuffer(numBytes);
-      Optional.ofNullable(CongestionController.instance())
-          .ifPresent(
-              congestionController ->
-                  congestionController.produceBytes(diskFileInfo.getUserIdentifier(), numBytes));
+      if (userBufferInfo != null) {
+        userBufferInfo.updateInfo(
+            System.currentTimeMillis(), new BufferStatusHub.BufferStatusNode(numBytes));
+      }
     }
 
     synchronized (flushLock) {
@@ -339,6 +353,7 @@ public abstract class PartitionDataWriter implements DeviceObserver {
     if (createFileResult._4() != null) {
       this.diskFileInfo = createFileResult._3();
       this.flusher = createFileResult._2();
+      this.flushWorkerIndex = this.flusher.getWorkerIndex();
 
       isMemoryShuffleFile.set(false);
       initFileChannelsForDiskFile();
