@@ -20,9 +20,12 @@ package org.apache.celeborn.service.deploy.worker
 import java.io.{FileNotFoundException, IOException}
 import java.nio.charset.StandardCharsets
 import java.util
+import java.util.concurrent.{Future => JFuture}
 import java.util.concurrent.atomic.AtomicBoolean
 import java.util.function.Consumer
 
+import scala.collection.JavaConverters._
+
 import com.google.common.base.Throwables
 import com.google.protobuf.GeneratedMessageV3
 import io.netty.util.concurrent.{Future, GenericFutureListener}
@@ -31,13 +34,14 @@ import org.apache.celeborn.common.CelebornConf
 import org.apache.celeborn.common.CelebornConf.MAX_CHUNKS_BEING_TRANSFERRED
 import org.apache.celeborn.common.exception.CelebornIOException
 import org.apache.celeborn.common.internal.Logging
-import org.apache.celeborn.common.meta.{FileInfo, FileManagedBuffers}
-import org.apache.celeborn.common.network.buffer.NioManagedBuffer
+import org.apache.celeborn.common.meta.{DiskFileInfo, FileInfo, MapFileMeta, MemoryFileInfo, ReduceFileMeta}
+import org.apache.celeborn.common.network.buffer.{FileChunkBuffers, MemoryChunkBuffers, NioManagedBuffer}
 import org.apache.celeborn.common.network.client.{RpcResponseCallback, TransportClient}
 import org.apache.celeborn.common.network.protocol._
 import org.apache.celeborn.common.network.server.BaseMessageHandler
 import org.apache.celeborn.common.network.util.{NettyUtils, TransportConf}
-import org.apache.celeborn.common.protocol.{MessageType, PartitionType, PbBufferStreamEnd, PbChunkFetchRequest, PbOpenStream, PbReadAddCredit, PbStreamHandler, StreamType}
+import org.apache.celeborn.common.protocol.{MessageType, PbBufferStreamEnd, PbChunkFetchRequest, PbOpenStream, PbOpenStreamList, PbOpenStreamListResponse, PbReadAddCredit, PbStreamHandler, PbStreamHandlerOpt, StreamType}
+import org.apache.celeborn.common.protocol.message.StatusCode
 import org.apache.celeborn.common.util.{ExceptionUtils, Utils}
 import org.apache.celeborn.service.deploy.worker.storage.{ChunkStreamManager, CreditStreamManager, PartitionFilesSorter, StorageManager}
 
@@ -60,11 +64,11 @@ class FetchHandler(
   var registered: Option[AtomicBoolean] = None
 
   def init(worker: Worker): Unit = {
-    workerSource.addGauge(WorkerSource.CHUNK_STREAM_COUNT) { () =>
+    workerSource.addGauge(WorkerSource.ACTIVE_CHUNK_STREAM_COUNT) { () =>
       chunkStreamManager.getStreamsCount
     }
 
-    workerSource.addGauge(WorkerSource.CREDIT_STREAM_COUNT) { () =>
+    workerSource.addGauge(WorkerSource.ACTIVE_CREDIT_STREAM_COUNT) { () =>
       creditStreamManager.getStreamsCount
     }
 
@@ -136,6 +140,33 @@ class FetchHandler(
           isLegacy = false,
           openStream.getReadLocalShuffle,
           callback)
+      case openStreamList: PbOpenStreamList =>
+        val shuffleKey = openStreamList.getShuffleKey()
+        val files = openStreamList.getFileNameList
+        val startIndices = openStreamList.getStartIndexList
+        val endIndices = openStreamList.getEndIndexList
+        val readLocalFlags = openStreamList.getReadLocalShuffleList
+        val pbOpenStreamListResponse = PbOpenStreamListResponse.newBuilder()
+        checkAuth(client, Utils.splitShuffleKey(shuffleKey)._1)
+        0 until files.size() foreach { idx =>
+          val pbStreamHandlerOpt = handleReduceOpenStreamInternal(
+            client,
+            shuffleKey,
+            files.get(idx),
+            startIndices.get(idx),
+            endIndices.get(idx),
+            readLocalFlags.get(idx))
+          if (pbStreamHandlerOpt.getStatus != StatusCode.SUCCESS.getValue) {
+            workerSource.incCounter(WorkerSource.OPEN_STREAM_FAIL_COUNT)
+          }
+          pbOpenStreamListResponse.addStreamHandlerOpt(pbStreamHandlerOpt)
+        }
+
+        client.getChannel.writeAndFlush(new RpcResponse(
+          rpcRequest.requestId,
+          new NioManagedBuffer(new TransportMessage(
+            MessageType.BATCH_OPEN_STREAM_RESPONSE,
+            pbOpenStreamListResponse.build().toByteArray).toByteBuffer)))
       case bufferStreamEnd: PbBufferStreamEnd =>
         handleEndStreamFromClient(
           client,
@@ -197,6 +228,100 @@ class FetchHandler(
 
   }
 
+  private def handleReduceOpenStreamInternal(
+      client: TransportClient,
+      shuffleKey: String,
+      fileName: String,
+      startIndex: Int,
+      endIndex: Int,
+      readLocalShuffle: Boolean = false): PbStreamHandlerOpt = {
+    try {
+      logDebug(s"Received open stream request $shuffleKey $fileName $startIndex " +
+        s"$endIndex get file name $fileName from client channel " +
+        s"${NettyUtils.getRemoteAddress(client.getChannel)}")
+
+      var fileInfo = getRawFileInfo(shuffleKey, fileName)
+      val streamId = chunkStreamManager.nextStreamId()
+      // we must get sorted fileInfo for the following cases.
+      // 1. when the current request is a non-range openStream, but the original unsorted file
+      //    has been deleted by another range's openStream request.
+      // 2. when the current request is a range openStream request.
+      if ((endIndex != Int.MaxValue) || (endIndex == Int.MaxValue
+          && !fileInfo.addStream(streamId))) {
+        fileInfo = partitionsSorter.getSortedFileInfo(
+          shuffleKey,
+          fileName,
+          fileInfo,
+          startIndex,
+          endIndex)
+      }
+      val meta = fileInfo.getReduceFileMeta
+      val streamHandler =
+        if (readLocalShuffle && !fileInfo.isInstanceOf[MemoryFileInfo]) {
+          chunkStreamManager.registerStream(
+            streamId,
+            shuffleKey,
+            fileName)
+          makeStreamHandler(
+            streamId,
+            meta.getNumChunks,
+            meta.getChunkOffsets,
+            fileInfo.asInstanceOf[DiskFileInfo].getFilePath)
+        } else fileInfo match {
+          case info: DiskFileInfo if info.isHdfs =>
+            chunkStreamManager.registerStream(
+              streamId,
+              shuffleKey,
+              fileName)
+            makeStreamHandler(streamId, numChunks = 0)
+          case _ =>
+            val managedBuffer = fileInfo match {
+              case df: DiskFileInfo =>
+                new FileChunkBuffers(df, transportConf)
+              case mf: MemoryFileInfo =>
+                new MemoryChunkBuffers(mf)
+            }
+            val fetchTimeMetric =
+              fileInfo match {
+                case info: DiskFileInfo =>
+                  storageManager.getFetchTimeMetric(info.getFile)
+                case _ =>
+                  null
+              }
+            chunkStreamManager.registerStream(
+              streamId,
+              shuffleKey,
+              managedBuffer,
+              fileName,
+              fetchTimeMetric)
+            if (meta.getNumChunks == 0)
+              logDebug(s"StreamId $streamId, fileName $fileName, mapRange " +
+                s"[$startIndex-$endIndex] is empty. Received from client channel " +
+                s"${NettyUtils.getRemoteAddress(client.getChannel)}")
+            else logDebug(
+              s"StreamId $streamId, fileName $fileName, numChunks ${meta.getNumChunks}, " +
+                s"mapRange [$startIndex-$endIndex]. Received from client channel " +
+                s"${NettyUtils.getRemoteAddress(client.getChannel)}")
+            makeStreamHandler(
+              streamId,
+              meta.getNumChunks)
+        }
+      workerSource.incCounter(WorkerSource.OPEN_STREAM_SUCCESS_COUNT)
+      PbStreamHandlerOpt.newBuilder().setStreamHandler(streamHandler)
+        .setStatus(StatusCode.SUCCESS.getValue)
+        .build()
+    } catch {
+      case e: IOException =>
+        val msg =
+          s"Read file: $fileName with shuffleKey: $shuffleKey error from ${NettyUtils.getRemoteAddress(
+            client.getChannel)}, Exception: ${e.getMessage}"
+        PbStreamHandlerOpt.newBuilder().setStatus(StatusCode.OPEN_STREAM_FAILED.getValue)
+          .setErrorMsg(msg).build()
+    } finally {
+      workerSource.stopTimer(WorkerSource.OPEN_STREAM_TIME, shuffleKey)
+    }
+  }
+
   private def handleOpenStreamInternal(
       client: TransportClient,
       shuffleKey: String,
@@ -208,71 +333,35 @@ class FetchHandler(
       isLegacy: Boolean,
       readLocalShuffle: Boolean = false,
       callback: RpcResponseCallback): Unit = {
+    checkAuth(client, Utils.splitShuffleKey(shuffleKey)._1)
     workerSource.recordAppActiveConnection(client, shuffleKey)
     workerSource.startTimer(WorkerSource.OPEN_STREAM_TIME, shuffleKey)
     try {
-      var fileInfo = getRawFileInfo(shuffleKey, fileName)
-      fileInfo.getPartitionType match {
-        case PartitionType.REDUCE =>
-          logDebug(s"Received open stream request $shuffleKey $fileName $startIndex " +
-            s"$endIndex get file name $fileName from client channel " +
-            s"${NettyUtils.getRemoteAddress(client.getChannel)}")
-
-          val streamId = chunkStreamManager.nextStreamId()
-          // we must get sorted fileInfo for the following cases.
-          // 1. when the current request is a non-range openStream, but the original unsorted file
-          //    has been deleted by another range's openStream request.
-          // 2. when the current request is a range openStream request.
-          if ((endIndex != Int.MaxValue) || (endIndex == Int.MaxValue && !fileInfo.addStream(
-              streamId))) {
-            fileInfo = partitionsSorter.getSortedFileInfo(
-              shuffleKey,
-              fileName,
-              fileInfo,
-              startIndex,
-              endIndex)
-          }
-          if (readLocalShuffle) {
-            chunkStreamManager.registerStream(
-              streamId,
-              shuffleKey,
-              fileName)
-            replyStreamHandler(
+      val fileInfo = getRawFileInfo(shuffleKey, fileName)
+      fileInfo.getFileMeta match {
+        case _: ReduceFileMeta =>
+          val pbStreamHandlerOpt =
+            handleReduceOpenStreamInternal(
               client,
-              rpcRequestId,
-              streamId,
-              fileInfo.numChunks(),
-              isLegacy,
-              fileInfo.getChunkOffsets,
-              fileInfo.getFilePath)
-          } else if (fileInfo.isHdfs) {
-            chunkStreamManager.registerStream(
-              streamId,
               shuffleKey,
-              fileName)
-            replyStreamHandler(client, rpcRequestId, streamId, numChunks = 0, isLegacy)
-          } else {
-            chunkStreamManager.registerStream(
-              streamId,
-              shuffleKey,
-              new FileManagedBuffers(fileInfo, transportConf),
               fileName,
-              storageManager.getFetchTimeMetric(fileInfo.getFile))
-            if (fileInfo.numChunks == 0)
-              logDebug(s"StreamId $streamId, fileName $fileName, mapRange " +
-                s"[$startIndex-$endIndex] is empty. Received from client channel " +
-                s"${NettyUtils.getRemoteAddress(client.getChannel)}")
-            else logDebug(
-              s"StreamId $streamId, fileName $fileName, numChunks ${fileInfo.numChunks}, " +
-                s"mapRange [$startIndex-$endIndex]. Received from client channel " +
-                s"${NettyUtils.getRemoteAddress(client.getChannel)}")
-            replyStreamHandler(client, rpcRequestId, streamId, fileInfo.numChunks(), isLegacy)
+              startIndex,
+              endIndex,
+              readLocalShuffle)
+
+          if (pbStreamHandlerOpt.getStatus != StatusCode.SUCCESS.getValue) {
+            throw new CelebornIOException(pbStreamHandlerOpt.getErrorMsg)
           }
-        case PartitionType.MAP =>
+          replyStreamHandler(client, rpcRequestId, pbStreamHandlerOpt.getStreamHandler, isLegacy)
+        case _: MapFileMeta =>
           val creditStreamHandler =
             new Consumer[java.lang.Long] {
               override def accept(streamId: java.lang.Long): Unit = {
-                replyStreamHandler(client, rpcRequestId, streamId, 0, isLegacy)
+                val pbStreamHandler = PbStreamHandler.newBuilder
+                  .setStreamId(streamId)
+                  .setNumChunks(0)
+                  .build()
+                replyStreamHandler(client, rpcRequestId, pbStreamHandler, isLegacy)
               }
             }
 
@@ -283,8 +372,7 @@ class FetchHandler(
             initialCredit,
             startIndex,
             endIndex,
-            fileInfo)
-        case PartitionType.MAPGROUP =>
+            fileInfo.asInstanceOf[DiskFileInfo])
       }
       workerSource.incCounter(WorkerSource.OPEN_STREAM_SUCCESS_COUNT)
     } catch {
@@ -296,28 +384,34 @@ class FetchHandler(
     }
   }
 
+  private def makeStreamHandler(
+      streamId: Long,
+      numChunks: Int,
+      offsets: util.List[java.lang.Long] = null,
+      filepath: String = ""): PbStreamHandler = {
+    val pbStreamHandlerBuilder = PbStreamHandler.newBuilder.setStreamId(streamId).setNumChunks(
+      numChunks)
+    if (offsets != null) {
+      pbStreamHandlerBuilder.addAllChunkOffsets(offsets)
+    }
+    if (filepath.nonEmpty) {
+      pbStreamHandlerBuilder.setFullPath(filepath)
+    }
+    pbStreamHandlerBuilder.build()
+  }
+
   private def replyStreamHandler(
       client: TransportClient,
       requestId: Long,
-      streamId: Long,
-      numChunks: Int,
-      isLegacy: Boolean,
-      offsets: util.List[java.lang.Long] = null,
-      filepath: String = ""): Unit = {
+      pbStreamHandler: PbStreamHandler,
+      isLegacy: Boolean): Unit = {
     if (isLegacy) {
       client.getChannel.writeAndFlush(new RpcResponse(
         requestId,
-        new NioManagedBuffer(new StreamHandle(streamId, numChunks).toByteBuffer)))
+        new NioManagedBuffer(new StreamHandle(
+          pbStreamHandler.getStreamId,
+          pbStreamHandler.getNumChunks).toByteBuffer)))
     } else {
-      val pbStreamHandlerBuilder = PbStreamHandler.newBuilder.setStreamId(streamId).setNumChunks(
-        numChunks)
-      if (offsets != null) {
-        pbStreamHandlerBuilder.addAllChunkOffsets(offsets)
-      }
-      if (filepath.nonEmpty) {
-        pbStreamHandlerBuilder.setFullPath(filepath)
-      }
-      val pbStreamHandler = pbStreamHandlerBuilder.build()
       client.getChannel.writeAndFlush(new RpcResponse(
         requestId,
         new NioManagedBuffer(new TransportMessage(
@@ -360,7 +454,8 @@ class FetchHandler(
       streamType: StreamType): Unit = {
     streamType match {
       case StreamType.ChunkStream =>
-        val (shuffleKey, fileName) = chunkStreamManager.getShuffleKeyAndFileName(streamId)
+        val streamState = chunkStreamManager.getStreamState(streamId)
+        val (shuffleKey, fileName) = (streamState.shuffleKey, streamState.fileName)
         workerSource.recordAppActiveConnection(client, shuffleKey)
         getRawFileInfo(shuffleKey, fileName).closeStream(streamId)
       case StreamType.CreditStream =>
