@@ -32,7 +32,7 @@ import io.netty.buffer.ByteBuf
 
 import org.apache.celeborn.common.exception.{AlreadyClosedException, CelebornIOException}
 import org.apache.celeborn.common.internal.Logging
-import org.apache.celeborn.common.meta.{DiskStatus, WorkerInfo, WorkerPartitionLocationInfo}
+import org.apache.celeborn.common.meta.{DiskFileInfo, DiskStatus, WorkerInfo, WorkerPartitionLocationInfo}
 import org.apache.celeborn.common.metrics.source.Source
 import org.apache.celeborn.common.network.buffer.{NettyManagedBuffer, NioManagedBuffer}
 import org.apache.celeborn.common.network.client.{RpcResponseCallback, TransportClient, TransportClientFactory}
@@ -43,9 +43,9 @@ import org.apache.celeborn.common.protocol.{PartitionLocation, PartitionSplitMod
 import org.apache.celeborn.common.protocol.PbPartitionLocation.Mode
 import org.apache.celeborn.common.protocol.message.StatusCode
 import org.apache.celeborn.common.unsafe.Platform
-import org.apache.celeborn.common.util.{DiskUtils, Utils}
+import org.apache.celeborn.common.util.{DiskUtils, ExceptionUtils, Utils}
 import org.apache.celeborn.service.deploy.worker.congestcontrol.CongestionController
-import org.apache.celeborn.service.deploy.worker.storage.{FileWriter, HdfsFlusher, LocalFlusher, MapPartitionFileWriter, StorageManager}
+import org.apache.celeborn.service.deploy.worker.storage.{HdfsFlusher, LocalFlusher, MapPartitionDataWriter, PartitionDataWriter, StorageManager}
 
 class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler with Logging {
 
@@ -119,6 +119,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
           client,
           pushData,
           pushData.requestId,
+          pushData.shuffleKey,
           () => {
             val partitionType =
               shufflePartitionType.getOrDefault(pushData.shuffleKey, PartitionType.REDUCE)
@@ -143,6 +144,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
           client,
           pushMergedData,
           pushMergedData.requestId,
+          pushMergedData.shuffleKey,
           () =>
             handlePushMergedData(
               pushMergedData,
@@ -264,10 +266,10 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
 
     if (fileWriter.isClosed) {
       logWarning(
-        s"[handlePushData] FileWriter is already closed! File path ${fileWriter.getFileInfo.getFilePath}")
+        s"[handlePushData] FileWriter is already closed! File path ${fileWriter.getDiskFileInfo.getFilePath}")
       callbackWithTimer.onFailure(new CelebornIOException("File already closed!"))
       fileWriter.decrementPendingWrites()
-      return;
+      return
     }
     val writePromise = Promise[Unit]()
     // for primary, send data to replica
@@ -313,7 +315,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
                     Option(CongestionController.instance()) match {
                       case Some(congestionController) =>
                         if (congestionController.isUserCongested(
-                            fileWriter.getFileInfo.getUserIdentifier)) {
+                            fileWriter.getDiskFileInfo.getUserIdentifier)) {
                           // Check whether primary congest the data though the replicas doesn't congest
                           // it(the response is empty)
                           callbackWithTimer.onSuccess(
@@ -341,10 +343,14 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
               } else if (e.getMessage.startsWith(StatusCode.PUSH_DATA_TIMEOUT_REPLICA.name())) {
                 workerSource.incCounter(WorkerSource.REPLICATE_DATA_TIMEOUT_COUNT)
                 callbackWithTimer.onFailure(e)
-              } else {
+              } else if (ExceptionUtils.connectFail(e.getMessage)) {
                 workerSource.incCounter(WorkerSource.REPLICATE_DATA_CONNECTION_EXCEPTION_COUNT)
                 callbackWithTimer.onFailure(
                   new CelebornIOException(StatusCode.PUSH_DATA_CONNECTION_EXCEPTION_REPLICA))
+              } else {
+                workerSource.incCounter(WorkerSource.REPLICATE_DATA_FAIL_NON_CRITICAL_CAUSE_COUNT)
+                callbackWithTimer.onFailure(
+                  new CelebornIOException(StatusCode.PUSH_DATA_FAIL_NON_CRITICAL_CAUSE_REPLICA))
               }
             }
           }
@@ -388,7 +394,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
             Option(CongestionController.instance()) match {
               case Some(congestionController) =>
                 if (congestionController.isUserCongested(
-                    fileWriter.getFileInfo.getUserIdentifier)) {
+                    fileWriter.getDiskFileInfo.getUserIdentifier)) {
                   if (isPrimary) {
                     callbackWithTimer.onSuccess(
                       ByteBuffer.wrap(
@@ -531,7 +537,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
     val closedFileWriter = fileWriters.find(_.isClosed)
     if (closedFileWriter.isDefined) {
       logWarning(
-        s"[handlePushMergedData] FileWriter is already closed! File path ${closedFileWriter.get.getFileInfo.getFilePath}")
+        s"[handlePushMergedData] FileWriter is already closed! File path ${closedFileWriter.get.getDiskFileInfo.getFilePath}")
       callbackWithTimer.onFailure(new CelebornIOException("File already closed!"))
       fileWriters.foreach(_.decrementPendingWrites())
       return
@@ -575,7 +581,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
                     Option(CongestionController.instance()) match {
                       case Some(congestionController) if fileWriters.nonEmpty =>
                         if (congestionController.isUserCongested(
-                            fileWriters.head.getFileInfo.getUserIdentifier)) {
+                            fileWriters.head.getDiskFileInfo.getUserIdentifier)) {
                           // Check whether primary congest the data though the replicas doesn't congest
                           // it(the response is empty)
                           callbackWithTimer.onSuccess(
@@ -603,10 +609,14 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
               } else if (e.getMessage.startsWith(StatusCode.PUSH_DATA_TIMEOUT_REPLICA.name())) {
                 workerSource.incCounter(WorkerSource.REPLICATE_DATA_TIMEOUT_COUNT)
                 callbackWithTimer.onFailure(e)
-              } else {
+              } else if (ExceptionUtils.connectFail(e.getMessage)) {
                 workerSource.incCounter(WorkerSource.REPLICATE_DATA_CONNECTION_EXCEPTION_COUNT)
                 callbackWithTimer.onFailure(
                   new CelebornIOException(StatusCode.PUSH_DATA_CONNECTION_EXCEPTION_REPLICA))
+              } else {
+                workerSource.incCounter(WorkerSource.REPLICATE_DATA_FAIL_NON_CRITICAL_CAUSE_COUNT)
+                callbackWithTimer.onFailure(
+                  new CelebornIOException(StatusCode.PUSH_DATA_FAIL_NON_CRITICAL_CAUSE_REPLICA))
               }
             }
           }
@@ -647,7 +657,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
           Option(CongestionController.instance()) match {
             case Some(congestionController) if fileWriters.nonEmpty =>
               if (congestionController.isUserCongested(
-                  fileWriters.head.getFileInfo.getUserIdentifier)) {
+                  fileWriters.head.getDiskFileInfo.getUserIdentifier)) {
                 if (isPrimary) {
                   callbackWithTimer.onSuccess(
                     ByteBuffer.wrap(
@@ -674,8 +684,8 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
    */
   private def getFileWriters(
       partitionIdToLocations: Array[(String, PartitionLocation)])
-      : (Array[FileWriter], Option[Int]) = {
-    val fileWriters = new Array[FileWriter](partitionIdToLocations.length)
+      : (Array[PartitionDataWriter], Option[Int]) = {
+    val fileWriters = new Array[PartitionDataWriter](partitionIdToLocations.length)
     var i = 0
     var exceptionFileWriterIndex: Option[Int] = None
     while (i < partitionIdToLocations.length) {
@@ -740,8 +750,10 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
       client: TransportClient,
       message: RequestMessage,
       requestId: Long,
+      shuffleKey: String,
       handler: () => Unit,
       callback: RpcResponseCallback): Unit = {
+    checkAuth(client, Utils.splitShuffleKey(shuffleKey)._1)
     try {
       handler()
     } catch {
@@ -792,7 +804,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
     val fileWriter =
       getFileWriterAndCheck(pushData.`type`(), location, isPrimary, callback) match {
         case (true, _) => return
-        case (false, f: FileWriter) => f
+        case (false, f: PartitionDataWriter) => f
       }
 
     // for mappartition we will not check whether disk full or split partition
@@ -801,10 +813,10 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
 
     if (fileWriter.isClosed) {
       logWarning(
-        s"[handleMapPartitionPushData] FileWriter is already closed! File path ${fileWriter.getFileInfo.getFilePath}")
+        s"[handleMapPartitionPushData] FileWriter is already closed! File path ${fileWriter.getDiskFileInfo.getFilePath}")
       callback.onFailure(new CelebornIOException("File already closed!"))
       fileWriter.decrementPendingWrites()
-      return;
+      return
     }
     val writePromise = Promise[Unit]()
     writeLocalData(Seq(fileWriter), body, shuffleKey, isPrimary, None, writePromise)
@@ -835,6 +847,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
       client,
       rpcRequest,
       requestId,
+      shuffleKey,
       () =>
         handleMapPartitionRpcRequestCore(
           requestId,
@@ -854,7 +867,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
   }
 
   private def mapPartitionRpcRequest(rpcRequest: RpcRequest)
-      : Tuple8[GeneratedMessageV3, Message, Boolean, Type, Mode, String, String, Boolean] = {
+      : (GeneratedMessageV3, Message, Boolean, Type, Mode, String, String, Boolean) = {
     try {
       val msg = TransportMessage.fromByteBuffer(
         rpcRequest.body().nioByteBuffer()).getParsedPayload.asInstanceOf[GeneratedMessageV3]
@@ -985,13 +998,13 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
     val fileWriter =
       getFileWriterAndCheck(messageType, location, isPrimary, callback) match {
         case (true, _) => return
-        case (false, f: FileWriter) => f
+        case (false, f: PartitionDataWriter) => f
       }
 
     // During worker shutdown, worker will return HARD_SPLIT for all existed partition.
     // This should before return exception to make current push request revive and retry.
     val isPartitionSplitEnabled = fileWriter.asInstanceOf[
-      MapPartitionFileWriter].getFileInfo.isPartitionSplitEnabled
+      MapPartitionDataWriter].getDiskFileInfo.isPartitionSplitEnabled
 
     if (shutdown.get() && (messageType == Type.REGION_START || messageType ==
         Type.PUSH_DATA_HAND_SHAKE) && isPartitionSplitEnabled) {
@@ -1019,7 +1032,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
               (
                 pbMsg.asInstanceOf[PbPushDataHandShake].getNumPartitions,
                 pbMsg.asInstanceOf[PbPushDataHandShake].getBufferSize)
-          fileWriter.asInstanceOf[MapPartitionFileWriter].pushDataHandShake(
+          fileWriter.asInstanceOf[MapPartitionDataWriter].pushDataHandShake(
             numPartitions,
             bufferSize)
         case Type.REGION_START =>
@@ -1032,11 +1045,11 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
               (
                 pbMsg.asInstanceOf[PbRegionStart].getCurrentRegionIndex,
                 pbMsg.asInstanceOf[PbRegionStart].getIsBroadcast)
-          fileWriter.asInstanceOf[MapPartitionFileWriter].regionStart(
+          fileWriter.asInstanceOf[MapPartitionDataWriter].regionStart(
             currentRegionIndex,
             isBroadcast)
         case Type.REGION_FINISH =>
-          fileWriter.asInstanceOf[MapPartitionFileWriter].regionFinish()
+          fileWriter.asInstanceOf[MapPartitionDataWriter].regionFinish()
         case _ => throw new IllegalArgumentException(s"Not support $messageType yet")
       }
       // for primary , send data to replica
@@ -1080,6 +1093,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
     }
 
     override def onFailure(e: Throwable): Unit = {
+      workerSource.stopTimer(workerSourceTime, s"$requestId")
       if (location != null) {
         logError(s"[handle$messageType.onFailure] partitionLocation: $location")
       }
@@ -1129,7 +1143,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
   private def checkFileWriterException(
       messageType: Message.Type,
       isPrimary: Boolean,
-      fileWriter: FileWriter,
+      fileWriter: PartitionDataWriter,
       callback: RpcResponseCallback): Unit = {
     logWarning(
       s"[handle$messageType] fileWriter $fileWriter has Exception ${fileWriter.getException}")
@@ -1160,7 +1174,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
       messageType: Message.Type,
       location: PartitionLocation,
       isPrimary: Boolean,
-      callback: RpcResponseCallback): (Boolean, FileWriter) = {
+      callback: RpcResponseCallback): (Boolean, PartitionDataWriter) = {
     val fileWriter = location.asInstanceOf[WorkingPartition].getFileWriter
     val exception = fileWriter.getException
     if (exception != null) {
@@ -1170,8 +1184,8 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
     (false, fileWriter)
   }
 
-  private def checkDiskFull(fileWriter: FileWriter): Boolean = {
-    if (fileWriter.flusher.isInstanceOf[HdfsFlusher]) {
+  private def checkDiskFull(fileWriter: PartitionDataWriter): Boolean = {
+    if (fileWriter.flusher == null || fileWriter.flusher.isInstanceOf[HdfsFlusher]) {
       return false
     }
     val mountPoint = fileWriter.flusher.asInstanceOf[LocalFlusher].mountPoint
@@ -1182,7 +1196,7 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
   }
 
   private def checkDiskFullAndSplit(
-      fileWriter: FileWriter,
+      fileWriter: PartitionDataWriter,
       isPrimary: Boolean,
       softSplit: AtomicBoolean,
       callback: RpcResponseCallback): Boolean = {
@@ -1192,28 +1206,34 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
          |CheckDiskFullAndSplit in
          |diskFull:$diskFull,
          |partitionSplitMinimumSize:$partitionSplitMinimumSize,
-         |splitThreshold:${fileWriter.getSplitThreshold()},
-         |fileLength:${fileWriter.getFileInfo.getFileLength}
-         |fileName:${fileWriter.getFileInfo.getFilePath}
+         |splitThreshold:${fileWriter.getSplitThreshold},
+         |fileLength:${fileWriter.getDiskFileInfo.getFileLength}
+         |fileName:${fileWriter.getDiskFileInfo.getFilePath}
          |""".stripMargin)
-    if (workerPartitionSplitEnabled && ((diskFull && fileWriter.getFileInfo.getFileLength > partitionSplitMinimumSize) ||
-        (isPrimary && fileWriter.getFileInfo.getFileLength > fileWriter.getSplitThreshold()))) {
-      if (softSplit != null && fileWriter.getSplitMode == PartitionSplitMode.SOFT &&
-        (fileWriter.getFileInfo.getFileLength < partitionSplitMaximumSize)) {
-        softSplit.set(true)
-      } else {
-        workerSource.incCounter(WorkerSource.WRITE_DATA_HARD_SPLIT_COUNT)
-        callback.onSuccess(ByteBuffer.wrap(Array[Byte](StatusCode.HARD_SPLIT.getValue)))
-        logTrace(
-          s"""
-             |CheckDiskFullAndSplit hardSplit
-             |diskFull:$diskFull,
-             |partitionSplitMinimumSize:$partitionSplitMinimumSize,
-             |splitThreshold:${fileWriter.getSplitThreshold()},
-             |fileLength:${fileWriter.getFileInfo.getFileLength},
-             |fileName:${fileWriter.getFileInfo.getFilePath}
-             |""".stripMargin)
-        return true
+    if (fileWriter.needHardSplitForMemoryShuffleStorage()) {
+      return true
+    }
+    val diskFileInfo = fileWriter.getDiskFileInfo
+    if (diskFileInfo != null) {
+      if (workerPartitionSplitEnabled && ((diskFull && diskFileInfo.getFileLength > partitionSplitMinimumSize) ||
+          (isPrimary && diskFileInfo.getFileLength > fileWriter.getSplitThreshold))) {
+        if (softSplit != null && fileWriter.getSplitMode == PartitionSplitMode.SOFT &&
+          (fileWriter.getDiskFileInfo.getFileLength < partitionSplitMaximumSize)) {
+          softSplit.set(true)
+        } else {
+          workerSource.incCounter(WorkerSource.WRITE_DATA_HARD_SPLIT_COUNT)
+          callback.onSuccess(ByteBuffer.wrap(Array[Byte](StatusCode.HARD_SPLIT.getValue)))
+          logTrace(
+            s"""
+               |CheckDiskFullAndSplit hardSplit
+               |diskFull:$diskFull,
+               |partitionSplitMinimumSize:$partitionSplitMinimumSize,
+               |splitThreshold:${fileWriter.getSplitThreshold},
+               |fileLength:${diskFileInfo.getFileLength},
+               |fileName:${diskFileInfo.getFilePath}
+               |""".stripMargin)
+          return true
+        }
       }
     }
     false
@@ -1228,13 +1248,13 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
   }
 
   private def writeLocalData(
-      fileWriters: Seq[FileWriter],
+      fileWriters: Seq[PartitionDataWriter],
       body: ByteBuf,
       shuffleKey: String,
       isPrimary: Boolean,
       batchOffsets: Option[Array[Int]],
       writePromise: Promise[Unit]): Unit = {
-    def writeData(fileWriter: FileWriter, body: ByteBuf, shuffleKey: String): Unit = {
+    def writeData(fileWriter: PartitionDataWriter, body: ByteBuf, shuffleKey: String): Unit = {
       try {
         fileWriter.write(body)
       } catch {
@@ -1265,10 +1285,10 @@ class PushDataHandler(val workerSource: WorkerSource) extends BaseMessageHandler
     batchOffsets match {
       case Some(batchOffsets) =>
         var index = 0
-        var fileWriter: FileWriter = null
+        var fileWriter: PartitionDataWriter = null
         while (index < fileWriters.length) {
-          fileWriter = fileWriters(index)
           if (!writePromise.isCompleted) {
+            fileWriter = fileWriters(index)
             val offset = body.readerIndex() + batchOffsets(index)
             val length =
               if (index == fileWriters.length - 1) {
