@@ -37,8 +37,8 @@ import org.apache.celeborn.service.deploy.worker.WorkerSource
 
 trait DeviceMonitor {
   def startCheck() {}
-  def registerFileWriter(fileWriter: FileWriter): Unit = {}
-  def unregisterFileWriter(fileWriter: FileWriter): Unit = {}
+  def registerFileWriter(fileWriter: PartitionDataWriter): Unit = {}
+  def unregisterFileWriter(fileWriter: PartitionDataWriter): Unit = {}
   // Only local flush needs device monitor.
   def registerFlusher(flusher: LocalFlusher): Unit = {}
   def unregisterFlusher(flusher: LocalFlusher): Unit = {}
@@ -94,7 +94,7 @@ class LocalDeviceMonitor(
           usage.freeSpace
         }
         workerSource.addGauge(WorkerSource.DEVICE_CELEBORN_TOTAL_CAPACITY, deviceLabel) { () =>
-          diskInfos.map(_.configuredUsableSpace).sum
+          diskInfos.map(_.totalSpace).sum
         }
         workerSource.addGauge(WorkerSource.DEVICE_CELEBORN_FREE_CAPACITY, deviceLabel) { () =>
           diskInfos.map(_.actualUsableSpace).sum
@@ -112,10 +112,10 @@ class LocalDeviceMonitor(
               val mountPoints = device.diskInfos.keySet.asScala.toList
               // tolerate time accuracy for better performance
               val now = System.currentTimeMillis()
-              for (concurrentList <- device.nonCriticalErrors.values().asScala) {
-                for (time <- concurrentList.asScala) {
+              for (concurrentSet <- device.nonCriticalErrors.values().asScala) {
+                for (time <- concurrentSet.asScala) {
                   if (now - time > device.notifyErrorExpireTimeout) {
-                    concurrentList.remove(time)
+                    concurrentSet.remove(time)
                   }
                 }
               }
@@ -158,17 +158,17 @@ class LocalDeviceMonitor(
           }
         }
       },
-      diskCheckInterval,
+      0,
       diskCheckInterval,
       TimeUnit.MILLISECONDS)
   }
 
-  override def registerFileWriter(fileWriter: FileWriter): Unit = {
+  override def registerFileWriter(fileWriter: PartitionDataWriter): Unit = {
     val mountPoint = DeviceInfo.getMountPoint(fileWriter.getFile.getAbsolutePath, diskInfos)
     observedDevices.get(diskInfos.get(mountPoint).deviceInfo).addObserver(fileWriter)
   }
 
-  override def unregisterFileWriter(fileWriter: FileWriter): Unit = {
+  override def unregisterFileWriter(fileWriter: PartitionDataWriter): Unit = {
     val mountPoint = DeviceInfo.getMountPoint(fileWriter.getFile.getAbsolutePath, diskInfos)
     observedDevices.get(diskInfos.get(mountPoint).deviceInfo).removeObserver(fileWriter)
   }
